//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxplayer.cpp,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2006/01/20 11:57:00 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



/*! \file mxplayer.cpp
    \brief <tt>MxPEG Player</tt>, the new <tt>MxPEG</tt> livestream viewer for 
    LINUX, MAC OS X and Windows.
*/



#include <mxcpcSimpleMxPEGViewer.h>
#include <mxcpcFastStreamSelectionDialog.h>
#include <mxcpcSDKInfoDialog.h> 
#include <mxcpcBestPerformanceMxPEGCodecFactory.h>
#include <mxcpcIPAddress.h>
#include <mxcpcProxyConfiguration.h>

#include <QApplication>

#include <cstdlib>
#include <cstdio>



#define MXPLAYER_APPNAME "MxPEG Player"



int main(int argc, char **argv) {

  QApplication qapp(argc, argv);
  
  mxcpcSimpleMxPEGViewer *viewer;
  mxcpcFastStreamSelectionDialog *faststream_selection_dialog;
  mxcpcSDKInfoDialog *sdk_info_dialog;
  mxcpcMxPEGCodecFactory *codec_factory;
  int ip1, ip2, ip3, ip4;
  
  codec_factory = new mxcpcBestPerformanceMxPEGCodecFactory();
  
  viewer = new mxcpcSimpleMxPEGViewer(0, 
                                      codec_factory,
                                      QString(MXPLAYER_APPNAME));
  viewer->show();
  
  faststream_selection_dialog = new mxcpcFastStreamSelectionDialog(0);
  faststream_selection_dialog->setWindowTitle(QString("Connect to Camera..."));
  
  sdk_info_dialog = new mxcpcSDKInfoDialog(0);
  sdk_info_dialog->show();
  sdk_info_dialog->hideLater();
  
  viewer->addMenuItem(QString("Connect to Camera..."), 
                      faststream_selection_dialog, SLOT(openDialog(void)));
  viewer->addMenuSeparator();
  viewer->addMenuItem(QString("About..."), 
                      sdk_info_dialog, SLOT(openDialog(void)));
  viewer->addMenuSeparator();
  viewer->addMenuItem(QString("Quit"), &qapp, SLOT(quit(void)));
  //viewer->setFramerateMeasureModeEnabled(true);
  
  QObject
   ::connect(faststream_selection_dialog, 
             SIGNAL(fastStreamSelected(const mxcpcIPAddress&, 
                                       int,
                                       const mxcpcProxyConfiguration&,
                                       float)),
             viewer,                    
             SLOT(playCameraLiveFastStream(const mxcpcIPAddress&,
                                           int,
                                           const mxcpcProxyConfiguration&,
                                           float)));
  
  // command arg?
  if(argc == 2) {
    ip1 = ip2 = ip3 = ip4 = 0;
    std::sscanf(argv[1], "%d.%d.%d.%d", &ip1, &ip2, &ip3, &ip4);
    viewer->playCameraLiveFastStream(mxcpcIPAddress(ip1, ip2, ip3, ip4),
                                     80,
                                     mxcpcProxyConfiguration(),   // no proxy
                                     8);
  }
  
  // have fun...
  qapp.exec();

  // shut down...
  delete viewer;
  delete faststream_selection_dialog;
  delete sdk_info_dialog;
  exit(0);
}
