//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcFramewiseMxPEGScanDecoder.h,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2005/12/20 16:41:55 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_FRAMEWISEMXPEGSCANDECODER_H__
#define __MXCPC_FRAMEWISEMXPEGSCANDECODER_H__



#include <mxcpcFramewiseMxPEGDecoder.h>



class mxcpcMxPEGDecoderBackEnd;



//! The framewise (frame prefetching) approach to <tt>MxPEG</tt> decoding 
//! provided by mxcpcFramewiseMxPEGDecoder needs an actual implementation of a 
//! <i>scan decoder</i> doing the actual bitstream crunching.
/*!
 *  Various implementations exist, for example the 
 *  mxcpcFramewiseMxPEGScanDecoderIPP which makes use of Intel's <i>Performance
 *  Primitives</i> library <i>(IPP)</i>.
 *
 *  \ingroup mxcpc_core
 */
class mxcpcFramewiseMxPEGScanDecoder {

 protected:
  static const unsigned char DefaultHuffmanTable_Y_DC[],
                             DefaultHuffmanTable_Y_AC[],
                             DefaultHuffmanTable_UV_DC[],
                             DefaultHuffmanTable_UV_AC[];
  static const int DefaultHuffmanTable_Y_DC_Size,
                   DefaultHuffmanTable_Y_AC_Size,
                   DefaultHuffmanTable_UV_DC_Size,
                   DefaultHuffmanTable_UV_AC_Size;

 protected:
  mxcpcMxPEGDecoderBackEnd *BackEnd;
                        
 public:
  mxcpcFramewiseMxPEGScanDecoder();
  virtual ~mxcpcFramewiseMxPEGScanDecoder();
  
 public:
  //! Returns a string elaborating on the type of acceleration method used. 
  virtual const char *getAccelerationTypeString(void) = 0;
  //! Informs the scan decoder about which decoder backend to feed with
  //! decoded data.
  /*!
   *  The scan decoder will <b>not</b> delete the backend upon its own 
   *  deletion.
   */
  void setBackEnd(mxcpcMxPEGDecoderBackEnd *backend);
  //! Copies 64 bytes from the specified location to the scan decoder's 
  //! Y quantization table. 
  virtual void setYQuantizationTable(unsigned char *data_bytes) = 0;
  //! Copies 64 bytes from the specified location to the scan decoder's 
  //! UV quantization table. 
  virtual void setUVQuantizationTable(unsigned char *data_bytes) = 0;
  //! Performs the scan decode for an entire <tt>MxPEG</tt> frame.
  /*!
   *  The frame descriptor is assumed to contain valid information. 
   *
   *  In case something goes wrong the backend is to be informed via a call 
   *  to its mxcpcMxPEGDecoderBackEnd::errorEncountered() method. 
   *
   *  \attention The mxcpcMxPEGDecoderBackEnd::frameComplete() method must
   *             <b>not</b> be called - this is done by the 
   *             mxcpcFramewiseMxPEGDecoder owning this scan decoder.
   *
   *  \attention Implementations must guard against missing backends.
   */
  virtual void performScanSweep(mxcpcFramewiseMxPEGDecoder::UndecodedFrameDescriptor
                                  *frame_descriptor) = 0;
};



#endif   // __MXCPC_FRAMEWISEMXPEGSCANDECODER_H__
