//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcMJPEGWriter.h,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2006/01/11 19:07:06 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_MJPEGWRITER_H__
#define __MXCPC_MJPEGWRITER_H__



#include <mxcpcJPEGReceiver.h>
#include <mxcpcBinaryFileWriter.h>

#include <cstdio>



//! Generates an <tt>MJPEG</tt> stream.
/*!
 *  \ingroup mxcpc_core
 *
 *  If invoked by specifying a filename to the constructor, the respective file
 *  is automatically opened for writing. It is also automatically closed again
 *  when the MJPEG writer is deleted later.
 *
 *  If, on the other hand, the MJPEG writer is invoked by specifying a 
 *  <tt>POSIX</tt> file descriptor to the constructor, the file descriptor is 
 *  used instead. Note that in this case the corresponing file is <b>not 
 *  closed</b> upon writer destruction.
 *
 *  If a separator string is specified to the constructor, the MJPEG writer will
 *  operate in HTTP support mode: before, between and after the frames 
 *  appropriate lines are inserted that allow for providing a HTTP MJPEG
 *  server push. 
 */
class mxcpcMJPEGWriter : public mxcpcJPEGReceiver,
                         public mxcpcBinaryFileWriter {

 private:
  int FrameNum;
  char *Separator;
  
 public:
  mxcpcMJPEGWriter(const char *filename,
                   const char *separator = 0);
  mxcpcMJPEGWriter(int file_descriptor,
                   const char *separator = 0);
  ~mxcpcMJPEGWriter();
 private:
  void installSeparator(const char *separator);
  
 public:
  //! (Re)implemented.
  void beginFrame(void);
  //! (Re)implemented.
  void endFrame(void);
  //! (Re)implemented.
  void receiveJPEGBytes(const mxcpc::u8 *buffer, int num);
};



#endif   // __MXCPC_MJPEGWRITER_H__
