//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcMxPEGDecoder.h,v $
//       ///       XXX       ///     $Revision: 1.3 $
//      ///      XXX XXX      ///     $Date: 2005/12/09 15:33:45 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_MXPEGDECODER_H__
#define __MXCPC_MXPEGDECODER_H__



#include <mxcpcStreamDecoder.h>



class mxcpcMxPEGDecoderBackEnd;



//! MxPEG stream decoder.
/*!
 *  \ingroup mxcpc_core
 *
 *  The new <tt>MxPEG</tt> decoder concept was designed to be portable and 
 *  modular. By <i>portable</i> we understand the codebase to be phrased 
 *  exclusively using standard C++ constructs, so that in contrast to the old 
 *  implementation that was infested virtually from head to toe with 
 *  <tt>MS Visual C++</tt> pecularities it will happily compile and run on all
 *  three major target platforms, being <tt>Windows</tt>, <tt>Linux</tt> and
 *  <tt>Mac OS X</tt>. By <i>modular</i> we refer to a clean object-oriented
 *  design, allowing the decoder to be easily configured and optimized for
 *  specific application settings by plugging together appropriate versions of
 *  decoding stages. A cellphone, for example, might be better off with a 
 *  decoder stage optimized for minimum memory consumtion rather than for 
 *  maximum decoder throughput, whereas workstation settings might rather enjoy
 *  the efficiency of a memory-hogging, all-out-performance version. 
 *
 *  To be more specific, the current <tt>MxPEG</tt> decoder implementation uses
 *  a three-stage approach:
 *  - The first stage is an implementation of a mxcpcStreamSource 
 *    representing - well - the source from where the decoder fetches the raw
 *    stream bytes. 
 *  - The second stage, the actual mxcpcMxPEGDecoder implementation, processes
 *    the raw stream data fetched from the stream source in a fashion that it 
 *    sees fit, and - as it goes along - emits signals containing decoded video 
 *    tiles and audio segments to the third stage.
 *  - The third and final stage, a specialized version of 
 *    mxcpcMxPEGDecoderBackEnd, accepts the aforementioned signals and decides
 *    what actually to do with the decoded video and audio data contained 
 *    therein. The point of the whole thing is that various interesting 
 *    scenarios are conceivable how in particular the decoded data can be 
 *    processed further, each of which allowing for its own ways of 
 *    streamlining the dataflow. To be most flexible in this respect, the 
 *    backend stage has also been given authority over the (memory) management
 *    of video and audio buffers. 
 *  
 *  \author Kai Hergenrther
 */
class mxcpcMxPEGDecoder : public mxcpcStreamDecoder {

 protected:
  mxcpcMxPEGDecoderBackEnd *BackEnd;

 public:
  //! The decoder assumes ownership over both stream source and backend 
  //! objects.
  mxcpcMxPEGDecoder(mxcpcStreamSource *source,
                    mxcpcMxPEGDecoderBackEnd *backend);
  ~mxcpcMxPEGDecoder();      
  
 public:
  //! Returns a pointer to the <tt>MxPEG</tt> decoder's backend.
  mxcpcMxPEGDecoderBackEnd *getBackEnd(void);
};



#endif   // __MXCPC_MXPEGDECODER_H__
