//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcMxPEGRawFrameDumper.h,v $
//       ///       XXX       ///     $Revision: 1.3 $
//      ///      XXX XXX      ///     $Date: 2006/01/13 13:53:08 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_MXPEGRAWFRAMEDUMPER_H__
#define __MXCPC_MXPEGRAWFRAMEDUMPER_H__



#include <mxcpcBufferedMxPEGDecoderBackEnd.h>



class mxcpcByteStreamReceiver;



//! <tt>MxPEG</tt> decoder backend dumping raw <tt>RGB</tt> / </tt>YUV</tt>
//! frames to a generic byte stream receiver.
/*!
 *   \ingroup mxcpc_core
 *
 *   Per default, the dumper writes raw <tt>RGB</tt> data with 3 bytes per pixel
 *   (being <tt>R</tt>, <tt>G</tt> and <tt>B</tt> values respectively). If 
 *   <tt>RGB</tt>-to-<tt>YUV</tt> conversion is turned on by passing the
 *   respective flag to the constructor, the frame is dumped in the planar
 *   <tt>yv12 YUV</tt> format. Note that this <tt>YUV</tt> format uses the
 *   <tt>YCbCr</tt> colorspace rather than plain <tt>YUV</tt>.
 *
 *   No frame separators will be written.
 *
 *   If <i>explicit dump mode</i> is enabled via setExplicitDumpModeEnabled(),
 *   frames will not automatically be dumped on frameComplete() events, but only
 *   when dumpFrame() gets called.
 *
 *   \note The generated raw frame formats are digestible by the commonly used 
 *   <tt>transcode</tt> and <tt>MPlayer/mencoder</tt> tools. 
 */
class mxcpcMxPEGRawFrameDumper : public mxcpcBufferedMxPEGDecoderBackEnd {

 private:
  mxcpcByteStreamReceiver *FrameReceiver;
  bool ConvertToYUV;
  mxcpc::u8 *YUVBuffer;
  bool ExplicitDumpMode;
  
 public:
  //! The frame dumper assumes ownership over the frame receiver.
  mxcpcMxPEGRawFrameDumper(mxcpcByteStreamReceiver *frame_receiver,
                           bool convert_to_YUV = false);
  ~mxcpcMxPEGRawFrameDumper();
  
 public:
  //! Explicitly causes the current contents of the frame buffer to be written
  //! to the frame receiver.
  void dumpFrame(void);
  //! Toggles explicit dump mode.
  void setExplicitDumpModeEnabled(bool enabled);
  //! (Re)implemented.
  void videoResolutionChanged(int width, int height);
  //! (Re)implemented.
  void errorEncountered(void);
  //! (Re)implemented.
  void frameComplete(void);
  //! (Re)implemented.
  void videoTileReceived(void);
};



#endif   // __MXCPC_MXPEGRAWFRAMEDUMPER_H__

