//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcStreamFile.h,v $
//       ///       XXX       ///     $Revision: 1.9 $
//      ///      XXX XXX      ///     $Date: 2006/01/16 15:37:00 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_STREAMFILE_H__
#define __MXCPC_STREAMFILE_H__



#include <mxcpcStreamSource.h>



//! File serving as stream source for stream decoding via mxcpcStreamDecoder.
/*!
 *  \ingroup mxcpc_core
 * 
 *  If invoked by specifying a filename to the constructor, the respective file
 *  is automatically opened. It is also automatically closed again when the 
 *  stream file is deleted later.
 *
 *  If, on the other hand, the stream file is invoked by specifying a 
 *  <tt>POSIX</tt> file descriptor to the constructor, the file descriptor is 
 *  used instead. Note that in this case the corresponing file is <b>not 
 *  closed</b> upon stream file destruction.
 *
 *  The stream file sports a so-called <i>Timed Block Mode</i> that can be
 *  toggled at any time using the setTimedBlockModeEnabled() method. If enabled,
 *  a UNIX <tt>select()</tt> call is issued prior to read attempts on the
 *  encapsuled file descriptor using the specified timeout. This causes the
 *  fetchBytes() implementation to block in the kernel if no data is available,
 *  but at most for that many milliseconds as was specified to 
 *  setTimedBlockModeEnabled(). If such a blocking times out before any new data
 *  come in over the file descriptor, fetchBytes() returns with a return value 
 *  of <tt>0</tt>.  
 *
 *  As usual, with <tt>Microsoft</tt> happily adding non-<tt>POSIX</tt> flags to
 *  <tt>POSIX</tt> interfaces, we have to provide a modified implementation for
 *  the <tt>WINDOWS</tt> platform.
 */
class mxcpcStreamFile : public mxcpcStreamSource {

 private:
  int InFile;
  bool StillUp;
  bool TimedBlockMode;
  long BlockTimeout;
  bool CloseFileUponDestruction;
  
 private:
  //! only used on WINDOWS
  int RestoreMode;
  long OSFHandle;
  
 public:
  mxcpcStreamFile(const char *filename);
  mxcpcStreamFile(int fd);
  ~mxcpcStreamFile();

 public:
  //! Toggles/configures the timed block mode, see mxcpcStreamFile.
  void setTimedBlockModeEnabled(bool enabled, long timeout = 1000);
  //! (Re)implemented.
  int fetchBytes(unsigned char *buffer, int num);
  //! (Re)implemented.
  bool stillUp(void);
};



#endif   // __MXCPC_STREAMFILE_H__
