//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcStreamSource.h,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2005/12/12 19:29:16 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_STREAMSOURCE_H__
#define __MXCPC_STREAMSOURCE_H__



//! Abstract representation of a (media) stream source.
/*!
 *  \ingroup mxcpc_core
 */
class mxcpcStreamSource {

 public:
  virtual ~mxcpcStreamSource();
  
 public:
  //! Fetches at most <tt>num</tt> raw stream bytes from the stream source,
  //! writes them to the specified buffer, and returns the number of bytes
  //! actually fetched.
  /*!
   *  A return value of <tt>0</tt> is not necessarily an error - maybe there
   *  simply were no more bytes available when the method was called.
   *
   *  However, in case of error, it is also <tt>0</tt> that is returned. 
   *
   *  In case of a return value of <tt>0</tt>,
   *  use the stillUp() method to test whether or not the stream source is
   *  still operational. If it is not, all subsequent calls to fetchBytes() will
   *  return <tt>0</tt>.
   */
  virtual int fetchBytes(unsigned char *buffer, int num) = 0;
  //! Returns whether or not the stream source is still operational.
  /*!
   *  If a stream source has become "not-up" once, it will never come up again.
   *  Instead, its fetchBytes() method will return <tt>0</tt> on all subsequent
   *  calls. Possible examples for a stream source that has become no longer up
   *  are an encountered EOF or read error.
   */
  virtual bool stillUp(void) = 0;
};



#endif   // __MXCPC_STREAMSOURCE_H__
