//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpc_exceptions.h,v $
//       ///       XXX       ///     $Revision: 1.3 $
//      ///      XXX XXX      ///     $Date: 2005/12/09 15:33:45 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_EXCEPTIONS_H__
#define __MXCPC_EXCEPTIONS_H__



#define MXCPC_RETHROW_UNHANDLED_EXCEPTION(e) \
         if(   !dynamic_cast<std::bad_alloc *>(&e) \
            && !dynamic_cast<mxcpcException *>(&e)) throw

            
            
#include <exception>
#include <memory>



//! Mother of all <tt>mxcpc</tt> exceptions :-)
/*!
 *  \ingroup mxcpc_core
 *
 *  As a matter of fact, mxcpcException itself is derived from 
 *  <tt>std::exception</tt>.
 *
 *  \attention
 *  It is policy throughout the <tt>mxcpc</tt> library, that whoever
 *  throws an mxcpcException guarantees preservation of application state
 *  integrity through proper rollback mechanisms. It is also convention that
 *  <tt>mxcpc</tt> methods including class constructors either fail with a 
 *  <tt>std::bad_alloc</tt> exception or with any kind of mxcpcException - 
 *  as a result, any other kind of exception that should escape 
 *  <tt>mxcpc</tt> code must be considered unexpected and no guarantees 
 *  about sane application state hold.
 */
class mxcpcException : public std::exception {};


//! Unspecific screw-up.
/*!
 *  \ingroup mxcpc_core
 */
class mxcpcDidntWorkOutException : public mxcpcException {};


//! Failure during file, network or similar operations.
/*!
 *  \ingroup mxcpc_core
 */
class mxcpcIOFailure : public mxcpcException {};


//! An operation was attempted that would have violated datastructure 
//! integrity. 
/*!
 *  \ingroup mxcpc_core
 *
 *  However, as it is with all mxcpcException s, datastructure integrity was
 *  preserved.
 */
class mxcpcDatastructureIntegrityViolation : public mxcpcException {};



#endif   // __MXCPC_EXCEPTIONS_H__
