//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpc_namespace.h,v $
//       ///       XXX       ///     $Revision: 1.8 $
//      ///      XXX XXX      ///     $Date: 2006/01/13 13:53:08 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_NAMESPACE_H__
#define __MXCPC_NAMESPACE_H__



class mxcpcStatusMessageHandler;



//! Namespace holding type definition stuff and general basic methods for the
//! <tt>mxcpc</tt> library.
/*!
 *  \ingroup mxcpc_core
 */
namespace mxcpc {

  typedef unsigned char  u8;
  typedef short         s16;
  typedef unsigned int  u32;
  
  extern mxcpcStatusMessageHandler *StatusMessageHandler;
  
  //! Base64 encode table, see RFC1113
  extern const char *Base64EncodeTable;
  
  //! Central definition of presets for camera framerate settings
  extern int FrameRatePresetNum;
  extern char *FrameRatePresetNames[];
  extern float FrameRatePresets[];
  
  //! Returns the mxcpc library's version string.
  const char *getVersionString(void); 
  
  //! Does the obvious thing.
  inline void clamp_s16(s16 *numba, s16 min, s16 max) {
    if(*numba < min) *numba = min;
    if(*numba > max) *numba = max;
  }
  //! Does the obvious thing.
  inline void clamp_int(int *numba, int min, int max) {
    if(*numba < min) *numba = min;
    if(*numba > max) *numba = max;
  }

  //! Endian conversion.
  inline void convert_endian_u32(mxcpc::u32 *numba_ptr) {
    mxcpc::u32 source_val;
    mxcpc::u8 *source_byte_ptr,
              *target_byte_ptr;
    
    source_val = *numba_ptr;
    source_byte_ptr = (mxcpc::u8 *)&source_val;
    target_byte_ptr = (mxcpc::u8 *)numba_ptr;
    
    target_byte_ptr[0] = source_byte_ptr[3];
    target_byte_ptr[1] = source_byte_ptr[2];
    target_byte_ptr[2] = source_byte_ptr[1];
    target_byte_ptr[3] = source_byte_ptr[0];
  }

  // Colorspace conversion from RGB to YUV.
  inline void convert_RGB_to_YUV(mxcpc::u8  r, mxcpc::u8  g, mxcpc::u8  b,
                                 mxcpc::u8 *y, mxcpc::u8 *u, mxcpc::u8 *v) {

    int int_r, int_g, int_b;
    int int_y, int_u, int_v;
                            
    int_r = (int)r;
    int_g = (int)g;
    int_b = (int)b;
    
    // coefficients computed from "Video Demystified" by K. JACK...
    int_y = (( 263*int_r + 516*int_g + 100*int_b) >> 10)   +    16;
    int_u = ((-152*int_r - 298*int_g + 450*int_b) >> 10)   +   128;
    int_v = (( 450*int_r - 377*int_g -  73*int_b) >> 10)   +   128;
    
    mxcpc::clamp_int(&int_y, 0, 255);
    mxcpc::clamp_int(&int_u, 0, 255);
    mxcpc::clamp_int(&int_v, 0, 255);
    
    *y = (mxcpc::u8)int_y;
    *u = (mxcpc::u8)int_u;
    *v = (mxcpc::u8)int_v;
  }
    
  // Colorspace conversion from RGB to YCbCr honoring ITU-R BT.601
  inline void convert_RGB_to_YCbCr(mxcpc::u8  r, mxcpc::u8   g, mxcpc::u8   b,
                                   mxcpc::u8 *y, mxcpc::u8 *Cb, mxcpc::u8 *Cr) {

    int int_r, int_g,  int_b;
    int int_y, int_Cb, int_Cr;
                            
    int_r = (int)r;
    int_g = (int)g;
    int_b = (int)b;
    
    // coefficients computed from ITU-R BT.601...
    int_y  = (( 306*int_r + 601*int_g + 117*int_b) >> 10);
    int_Cb = ((-173*int_r - 339*int_g + 512*int_b) >> 10)   +   128;
    int_Cr = (( 512*int_r - 429*int_g -  83*int_b) >> 10)   +   128;
    
    mxcpc::clamp_int(&int_y,  0, 255);
    mxcpc::clamp_int(&int_Cb, 0, 255);
    mxcpc::clamp_int(&int_Cr, 0, 255);
    
    *y  = (mxcpc::u8)int_y;
    *Cb = (mxcpc::u8)int_Cb;
    *Cr = (mxcpc::u8)int_Cr;
  }
  
  //! Encodes 3-byte block of binary input as 4-byte block of base64 characters.
  void encode_base64_block(unsigned char in[3], 
                           unsigned char out[4], 
                           int len);
  //! Encodes the given number of binary input bytes as a base64 character 
  //! string. 
  void encode_base64(unsigned char *data, int byte_num, 
                     unsigned char *target_buffer);   
  
  //! Used from within the <tt>mxcpc</tt> library to emit status messages.
  /*!
   *  That way, there is a central place that can be tweaked in order to change
   *  outputting of status messages.
   */ 
  void sendStatusMsg(const char *txt, void *class_ptr = 0);
  //! Registers the specified status message handler with <tt>mxcpc</tt>'s 
  //! central status messaging facility.
  /*!
   *  The <tt>mxcpc</tt> library assumes ownership over the specified handler
   *  object. Any potential previously installed message handler is discarded
   *  properly.
   *
   *  Passing <tt>0</tt> to the method will properly discard a previously 
   *  installed custom status message handler and revert to default status
   *  notification - which is sending all messages to <tt>stdout</tt>.
   *  Note that when shutting down, an <tt>mxcpc</tt> application should
   *  do exactly this to properly free the registered status message handler and
   *  the associated resources - if it had one installed in the first place.
   *
   *  Note that when using <tt>mxcpc</tt> functionality from different threads,
   *  any potentially installed custom status message handler must arrange for
   *  proper synchronization.
   */
  void setStatusMessageHandler(mxcpcStatusMessageHandler *handler);
};



#endif   // __MXCPC_NAMESPACE_H__

