//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcBinaryFileWriter_win.cpp,v $
//       ///       XXX       ///     $Revision: 1.1 $
//      ///      XXX XXX      ///     $Date: 2006/01/11 19:07:06 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcBinaryFileWriter.h>

#include <unistd.h>
#include <fcntl.h>



/*! 
 *  \file mxcpcBinaryFileWriter_win.cpp
 *
 *  \brief Alternate implementation for the <tt>WINDOWS</tt> platform
 * 
 *  Differs from the original <tt>POSIX</tt>-dependent implementation in that 
 *  we explicitly have to request binary mode when doing an <tt>open()</tt> or
 *  <tt>fopen()</tt> to avoid having our carriage returns (13s) implicitly 
 *  toasted.
 */



mxcpcBinaryFileWriter::mxcpcBinaryFileWriter(const char *filename) {

  OutFile = open(filename, 
                 O_CREAT | O_WRONLY | O_TRUNC | O_BINARY);
                                     
  if(OutFile != -1) {
    CloseFileUponDeletion = true;
    ErrorEncountered = false;
  }
  else {
    CloseFileUponDeletion = false;
    ErrorEncountered = true;
  }

  RestoreMode = -1;
}


mxcpcBinaryFileWriter::mxcpcBinaryFileWriter(int file_descriptor) {

  OutFile = file_descriptor;
  
  if((RestoreMode = setmode(file_descriptor, O_BINARY)) != -1)
   ErrorEncountered = false;
  else
   ErrorEncountered = true;
  
  CloseFileUponDeletion = false;
}


mxcpcBinaryFileWriter::~mxcpcBinaryFileWriter() {
  
  if(RestoreMode != -1) setmode(OutFile, RestoreMode);
  
  if(CloseFileUponDeletion) close(OutFile);
}



void mxcpcBinaryFileWriter::receiveStreamBytes(const mxcpc::u8 *bytes, 
                                               int byte_num)           {

  int written, to_write;
   
  if(ErrorEncountered || (OutFile < 0)) return;
  
  // have to do it tricky to properly handle short writes...
  to_write = byte_num;
  while(to_write > 0) { 
  
    written = write(OutFile, bytes, to_write);
    if(written == -1) {
      ErrorEncountered = true;
      break;
    }
    
    to_write -= written;
    bytes += written;
  }
}

