//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcCameraLiveMxPEGFastStreamPOSIX.cpp,v $
//       ///       XXX       ///     $Revision: 1.3 $
//      ///      XXX XXX      ///     $Date: 2005/12/19 17:38:39 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcCameraLiveMxPEGFastStreamPOSIX.h>
#include <mxcpc_exceptions.h>

#include <cstdio>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <fcntl.h>



mxcpcCameraLiveMxPEGFastStreamPOSIX
 ::mxcpcCameraLiveMxPEGFastStreamPOSIX(int ip1,
                                       int ip2, 
                                       int ip3, 
                                       int ip4, 
                                       int port,
                                       int fps,
                                       bool asynchronous)
    : mxcpcCameraLiveMxPEGFastStream(ip1,
                                     ip2,
                                     ip3,
                                     ip4,
                                     port,
                                     fps) {
                                                                         
  constructCommonStuff(mxcpcIPAddress(ip1, ip2, ip3, ip4), port,
                       (float)fps,
                       asynchronous);
}
   

mxcpcCameraLiveMxPEGFastStreamPOSIX
 ::mxcpcCameraLiveMxPEGFastStreamPOSIX(const mxcpcIPAddress& camera_ip, 
                                       int port,
                                       float fps,
                                       bool asynchronous)
    : mxcpcCameraLiveMxPEGFastStream(camera_ip.getIP1(),
                                     camera_ip.getIP2(),
                                     camera_ip.getIP3(),
                                     camera_ip.getIP4(),
                                     port,
                                     (int)fps)                         {
         
  constructCommonStuff(camera_ip, port,
                       fps,
                       asynchronous);
}


void mxcpcCameraLiveMxPEGFastStreamPOSIX
      ::constructCommonStuff(const mxcpcIPAddress& camera_ip, 
                             int port,
                             float fps,
                             bool asynchronous)               {         
                                                                                    
  in_addr_t server_ip;
  struct sockaddr_in server_socket_info;
  int flags;
  char txt[160], *txt_ptr;
  int len, written;
  
  
  Socket = -1;
  try {
  
    // get a socket
    Socket = socket(PF_INET, SOCK_STREAM, 0);
    if(Socket < 0) 
      throw mxcpcIOFailure();
    
    // connect to server...
    server_socket_info.sin_family = AF_INET;
    server_socket_info.sin_port   = htons(port);
    server_ip = 0;
    server_ip += camera_ip.getIP1();   server_ip *= 256;
    server_ip += camera_ip.getIP2();   server_ip *= 256;
    server_ip += camera_ip.getIP3();   server_ip *= 256;
    server_ip += camera_ip.getIP4();
    server_socket_info.sin_addr.s_addr = htonl(server_ip);
    if(::connect(Socket, (const struct sockaddr *)&server_socket_info,
                         (socklen_t)sizeof(server_socket_info))) 
      throw mxcpcIOFailure();
      
    // if requested, make socket nonblocking...
    if(asynchronous) {
      flags = fcntl(Socket, F_GETFL);
      flags |= O_NONBLOCK;
      fcntl(Socket, F_SETFL, flags);  
    }
      
    // do HTTP GET request 
    if(fps < 0.0f) fps = 4.0f;
    std::sprintf(txt, "GET /control/faststream.jpg?noaudio&fps=%f HTTP/1.0\n\n",
                 fps);
    len = std::strlen(txt);
    txt_ptr = txt;
    do {
      written = write(Socket, txt_ptr, len);
      if(written < 0) throw mxcpcIOFailure();
      len -= written;
      txt_ptr += written;
    } while(len);
      
  } catch(std::exception &e) {
    
      MXCPC_RETHROW_UNHANDLED_EXCEPTION(e);
      
      if(Socket >= 0) shutdown(Socket, SHUT_RDWR);
    
      throw;
    }
}


mxcpcCameraLiveMxPEGFastStreamPOSIX::~mxcpcCameraLiveMxPEGFastStreamPOSIX() {

  shutdown(Socket, SHUT_RDWR);
}



int mxcpcCameraLiveMxPEGFastStreamPOSIX::fetchBytes(unsigned char *buffer, 
                                                    int num)               {
                                                    
   int result;
   
   result = read(Socket, buffer, num);
   if(result < 0) result = 0;
   
   return(result);
}
