//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcFramewiseMxPEGDecoder.cpp,v $
//       ///       XXX       ///     $Revision: 1.6 $
//      ///      XXX XXX      ///     $Date: 2006/01/11 19:07:06 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcFramewiseMxPEGDecoder.h>
#include <mxcpcFramewiseMxPEGScanDecoder.h>
#include <mxcpcStreamSource.h>
#include <mxcpcMxPEGDecoderBackEnd.h>
#include <mxcpc_exceptions.h>
#include <mxcpc_namespace.h>

#include <exception>
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <typeinfo>



mxcpcFramewiseMxPEGDecoder
 ::mxcpcFramewiseMxPEGDecoder(mxcpcStreamSource *source,
                              mxcpcMxPEGDecoderBackEnd *backend,
                              mxcpcFramewiseMxPEGScanDecoder 
                              *scan_decoder)
    : mxcpcMxPEGDecoder(source, backend)                         {
  
  char txt[100], txt2[200];
            
  PrefetchBuffSize = 4 * 1024 * 1024;
  PrefetchedBytes = 0;
  
  PrefetchBuffer = 0;
  try {
  
    PrefetchBuffer = new unsigned char[PrefetchBuffSize];
    
  } catch(std::exception &e) {
    
      MXCPC_RETHROW_UNHANDLED_EXCEPTION(e);
      
      delete[] PrefetchBuffer;
      
      throw;
    }
  
  // --- init state machine datafields... ---
  
  State = AwaitSOI;
  
  CurrentBytePtr = 0;
  BytesLeft = 0;
  
  AwaitMarkerStage = 0;
  
  ReadMarkerStage = 0;
  ReadMarkerFirstLenByte = 0;
  MarkerBytesNeeded = 0;
  
  ScanStage = 0;
  
  MarkerPayload = 0;
  
  Width = Height = 0;
  ScanDecoder = scan_decoder;
  ScanDecoder->setBackEnd(backend);

  std::strncpy(txt, ScanDecoder->getAccelerationTypeString(), 100);
  txt[99] = '\0';
  std::sprintf(txt2, "FramewiseMxPEGDecoder : acceleration method : %s",
                     txt);
  mxcpc::sendStatusMsg(txt2);
}


mxcpcFramewiseMxPEGDecoder::~mxcpcFramewiseMxPEGDecoder() {

  delete[] PrefetchBuffer;
  delete   ScanDecoder;
}



int mxcpcFramewiseMxPEGDecoder::decode(int num) {

  int bytes_allowed;
  unsigned char *buff_ptr;
  int bytes_read, bytes_processed;
  
  // determine max no of bytes to process for this call...
  bytes_allowed = PrefetchBuffSize - PrefetchedBytes;
  if(bytes_allowed > num) bytes_allowed = num;
  if(bytes_allowed < 1) {
    mxcpc::sendStatusMsg("!!! MxPEG decoder failure (prefetch buffer overflow) !!!", this);
    std::exit(666);
    //return(0);
  }  
  
  bytes_processed = 0;
  while(bytes_allowed > 0) {
   
    buff_ptr = PrefetchBuffer + PrefetchedBytes;
    bytes_read = StreamSource->fetchBytes(buff_ptr, bytes_allowed); 
    if(!bytes_read) break;
    
    PrefetchedBytes += bytes_read;
   
    feedBytes(buff_ptr, bytes_read);   // might remove entire frame image and
                                       // thus modify _PrefetchedBytes_ variable
    
    bytes_processed += bytes_read;
    bytes_allowed   -= bytes_read;
  } 
  
  return(bytes_processed);
}


void mxcpcFramewiseMxPEGDecoder::feedBytes(unsigned char *buffer,
                                           int num) {
  
  unsigned char marker_type;
  int width, height;
  bool need_to_copy_down;

  if(num < 1) return;
  
  CurrentBytePtr = buffer;
  BytesLeft = num;
  while(BytesLeft) {
  
    need_to_copy_down = false;
  
    switch(State) {
    
      // all these states can digest BytesLeft == 0
    
      case AwaitSOI:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xd8)   // SOI marker 
          State = AwaitAPP0;
        else
          State = No_Good_State;
        break;
        
      case AwaitAPP0:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xe0)
          State = APP0;
        else
          State = No_Good_State;
        break;
        
      case APP0:
        if(!readMarkerPayload()) return;
        State = Determine_FingerPrint_Or_FrameHeader;
        break;
        
      case Determine_FingerPrint_Or_FrameHeader:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xfe) State = Determine_FingerPrint_Or_FrameHeader2;
        else                    State = No_Good_State;
        break;
        
      case Determine_FingerPrint_Or_FrameHeader2:
        if(!readMarkerPayload()) return;  
        if(!strncmp((char *)MarkerPayload, "#:M1IMG", 7)) 
          State = FingerPrint;
        else if(!strncmp((char *)MarkerPayload, "MXF", 3)) 
          State = FrameHeader;
        else
          State = No_Good_State;      
        break;
        
      case FingerPrint:
        State = AwaitFrameHeader;
        break;
         
      case AwaitFrameHeader:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xfe)
          State = AwaitFrameHeader2;
        else
          State = No_Good_State;
        break;
         
      case AwaitFrameHeader2:
        if(!readMarkerPayload()) return;  
        if(!strncmp((char *)MarkerPayload, "MXF", 3))
          State = FrameHeader;
        else
          State = No_Good_State; 
        break;
        
      case FrameHeader:
        State = BitMask_Or_Tables;
        break;
        
      case BitMask_Or_Tables:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xdb) {
          FrameDescriptor.TileBits = 0;
          State = QuantizationTable1;
        }
        else if(marker_type == 0xfe)
          State = BitMask;
        else
          State = No_Good_State;
        break;
        
      case QuantizationTable1:
        if(!readMarkerPayload()) return;
        ScanDecoder->setYQuantizationTable(&MarkerPayload[1]);
        State = AwaitQuantizationTable2;
        break;
        
      case AwaitQuantizationTable2:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xdb)
          State = QuantizationTable2;
        else
          State = No_Good_State;
        break;
     
      case QuantizationTable2:
        if(!readMarkerPayload()) return;
        ScanDecoder->setUVQuantizationTable(&MarkerPayload[1]);
        State = Huffman_Or_SOF;
        break;
     
      case Huffman_Or_SOF:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xc4)
          State = HuffmanTable;
        else if(marker_type == 0xc0)
          State = SOF;
        else 
          State = No_Good_State;
        break;
        
      case SOF:
        if(!readMarkerPayload()) return;
        width  = (int)MarkerPayload[3]*256 + (int)MarkerPayload[4];
        height = (int)MarkerPayload[1]*256 + (int)MarkerPayload[2];
        FrameDescriptor.setResolution(width, height);
        State = Huffman_Or_Scan;
        break;
        
      case HuffmanTable:
        if(!readMarkerPayload()) return;
        State = Huffman_Or_Scan;
        break;
        
      case Huffman_Or_Scan:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xda)
          State = Scan;
        else if(marker_type == 0xc4)
          State = HuffmanTable;
        else 
          State = No_Good_State;
        break;
        
      case Scan:
        if(!readMarkerPayload()) return;
        FrameDescriptor.ScanBytes = CurrentBytePtr;
        State = Scan2;
        break;
        
      case Scan2:
        if(!doScan()) return;
        FrameDescriptor.ScanLength = CurrentBytePtr - FrameDescriptor.ScanBytes;
        State = Audio_Or_SOI;
        break;
        
      case BitMask:
        if(!readMarkerPayload()) return;
        FrameDescriptor.TileBits = &MarkerPayload[12];
        State = Tables_Or_Scan;
        break;
        
      case Tables_Or_Scan:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xdb)
          State = QuantizationTable1;
        else if(marker_type == 0xda)
          State = Scan;
        else
          State = No_Good_State;
        break;
        
      case Audio_Or_SOI:
        if(!awaitMarker(&marker_type)) return;
        if(marker_type == 0xed)
          State = Audio;
        else if(marker_type == 0xd8) {
          need_to_copy_down = true;
          State = AwaitAPP0;
        }
        else {
          need_to_copy_down = true;
          State = No_Good_State;
        }
        break;
        
      case Audio:
        if(!readMarkerPayload()) return;
        need_to_copy_down = true;
        State = AwaitSOI;
        break;
        
      case No_Good_State:
        mxcpc::sendStatusMsg("!!! MxPEG decoder failure (no_good_state) !!!", this);
        std::exit(666);
        break;
        
      default:
        mxcpc::sendStatusMsg("!!! MxPEG decoder INTERNAL FAILURE !!!");
        std::exit(666);
        break;        
    }
    
    if(need_to_copy_down) {   
    
      // ---------- we've got one complete undecoded frame image ----------
  
      // check video resolution...
      if(   (FrameDescriptor.Width != Width)
         || (FrameDescriptor.Height != Height)) {
         
        BackEnd->videoResolutionChanged(FrameDescriptor.Width,
                                        FrameDescriptor.Height);
        Width  = FrameDescriptor.Width;
        Height = FrameDescriptor.Height;
      }

      ScanDecoder->performScanSweep(&FrameDescriptor);      
             
      BackEnd->frameComplete();
                                       
      // copy down remainder of prefetched bytes...
      if(BytesLeft) {
        std::memmove(PrefetchBuffer, CurrentBytePtr, BytesLeft);   
         // could overlap
      }
      CurrentBytePtr = PrefetchBuffer;
      PrefetchedBytes = BytesLeft;
        // _need_to_copy_down_ flag will be cleared first thing in the next
        // loop cycle...
        
      // prepare for new attack run...
      FrameDescriptor.reset(Width, Height);
    }
  }
}


/*!
 *  Eats BytesLeft==0 
 *
 *  Result:
 *  - <tt>false</tt> : no marker yet, all bytes consumed
 *  - <tt>true</tt> : marker found, BytesLeft and CurrentBytePtr modified 
 *    accordingly (bytes may be consumed), AwaitMarkerStage == 0
 */
bool mxcpcFramewiseMxPEGDecoder::awaitMarker(unsigned char *marker_type) {
  
  if(AwaitMarkerStage == 0) {   // nothing found yet
  
    while(BytesLeft) {
      if(*CurrentBytePtr == (unsigned char)0xff) {
        CurrentBytePtr++;
        BytesLeft--;
        if(BytesLeft) {
          *marker_type = *CurrentBytePtr;
          CurrentBytePtr++;
          BytesLeft--;
          return(true);   // found a marker!
        }
        else {
          // got the 0xff, but no byte left to determine marker type
          AwaitMarkerStage = 1;
          return(false);
        }
      }
      
      CurrentBytePtr++;
      BytesLeft--;
    }
    return(false);   // still nothing!
  }   // stage 0
  
  else {   // stage 1, still net type...
    if(BytesLeft) {
      *marker_type = *CurrentBytePtr;
      CurrentBytePtr++;
      BytesLeft--;
      AwaitMarkerStage = 0;
      return(true);
    }
    else 
      return(false);
  }
}


/*!
 *  If method returns <tt>true</tt>, the payload is recognized to be completely
 *  present and the <tt>MarkerPayload</tt> pointer is set accordingly.
 *
 *  Analogous behavior as waitMarker().
 */
bool mxcpcFramewiseMxPEGDecoder::readMarkerPayload(void) {

  for(;;) {
    
    if(ReadMarkerStage == 0) {   // got nothing
      if(!BytesLeft) return(false);
      else if(BytesLeft == 1) {
        ReadMarkerFirstLenByte = *CurrentBytePtr;
        CurrentBytePtr++;
        BytesLeft--;
        ReadMarkerStage = 1;
        return(false);
      }
      else {  
        MarkerBytesNeeded = 256*(int)CurrentBytePtr[0] 
                             + (int)CurrentBytePtr[1]
                             - 2;
        CurrentBytePtr += 2;
        BytesLeft -= 2;
        MarkerPayload = CurrentBytePtr;
        ReadMarkerStage = 2;
        continue;
      }
    }   // stage 0
    
    else if(ReadMarkerStage == 1) {   // got the 1 length byte, waiting for 2nd 
      if(!BytesLeft) return(false);
      MarkerBytesNeeded = 256*(int)ReadMarkerFirstLenByte 
                           + (int)*CurrentBytePtr
                           - 2;
      CurrentBytePtr++;
      BytesLeft--;
      MarkerPayload = CurrentBytePtr;
      ReadMarkerStage = 2;
      continue;                                
    }   // stage 1
    
    else if(ReadMarkerStage == 2) {   // reading marker payload 
      
      if(!BytesLeft) return(false);
      if(BytesLeft >= MarkerBytesNeeded) {
        CurrentBytePtr += MarkerBytesNeeded;
        BytesLeft -= MarkerBytesNeeded;
        MarkerBytesNeeded = 0;
        ReadMarkerFirstLenByte = 0;
        ReadMarkerStage = 0;
        return(true);
      }
      else {
        MarkerBytesNeeded -= BytesLeft;
        CurrentBytePtr += BytesLeft;
        BytesLeft = 0;
        return(false);
      }
    }
  }
}


/*!
 *  Analogous behavior as waitMarker().
 */
bool mxcpcFramewiseMxPEGDecoder::doScan(void) {

label_doScan_begin:

  if(ScanStage == 0) {   // nuffin'
    while(BytesLeft) {
      if(*CurrentBytePtr == 0xff) {
        CurrentBytePtr++;
        BytesLeft--;
        if(!BytesLeft) {
          ScanStage = 1;
          return(false);
        }
        else {
          if(*CurrentBytePtr == 0x00) {
            CurrentBytePtr++;
            BytesLeft--;
            continue;
          }
          else if(*CurrentBytePtr == 0xd9) {   // EOI marker
            CurrentBytePtr++;
            BytesLeft--;
            return(true);
          }
          else {
            mxcpc::sendStatusMsg("!!! MxPEG decoder failure "
                                 "(invalid scan termination) !!!");
            std::exit(666);
          }
        }
      }
      else {
        CurrentBytePtr++;
        BytesLeft--;
      }
    }
    return(false);
  }   // stage 0
  
  else {   // stage 1 - read a 0xff, decide whether marker or padding
    if(!BytesLeft) return(false);
    if(*CurrentBytePtr == 0x00) {
      CurrentBytePtr++;
      BytesLeft--;
      ScanStage = 0;
      goto label_doScan_begin;   // nasty, but what da hell...
    }
    else if(*CurrentBytePtr == 0xd9) {   // EOI marker
      CurrentBytePtr++;
      BytesLeft--;
      ScanStage = 0;
      return(true);
    }
    else {
      mxcpc::sendStatusMsg("!!! MxPEG decoder failure "
                           "(invalid scan termination) !!!");
      std::exit(666);
    }
  }   // stage 1
}

