//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcJPEGSequenceWriter.cpp,v $
//       ///       XXX       ///     $Revision: 1.2 $
//      ///      XXX XXX      ///     $Date: 2005/12/14 10:38:52 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcJPEGSequenceWriter.h>

#include <cstdio>
#include <cstring>
#include <memory>



mxcpcJPEGSequenceWriter::mxcpcJPEGSequenceWriter(const char *filename_prefix) {

  int prefix_len;

  prefix_len = std::strlen(filename_prefix);
  
  FileNamePrefix = 0;
  FileName       = 0;
  
  try {
    
    FileNamePrefix = new char[prefix_len + 1];
    FileName       = new char[prefix_len + 20 + 1];
                           // so we can append a sequence number...
  
  } catch(std::bad_alloc) {
    
      delete[] FileNamePrefix;
      delete[] FileName;
      
      throw;
    }
    
  std::strcpy(FileNamePrefix, filename_prefix);

  OutFile = 0;
  FrameNum = 0;
  
  ErrorEncountered = false;
}


mxcpcJPEGSequenceWriter::~mxcpcJPEGSequenceWriter() {

  delete[] FileNamePrefix;
  delete[] FileName;
  
  if(OutFile) std::fclose(OutFile);
}


void mxcpcJPEGSequenceWriter::beginFrame(void) {

  if(OutFile) {   // being paranoid
    
    std::fclose(OutFile);
    OutFile = 0;
    
    ErrorEncountered = true;
  }
  
  std::sprintf(FileName, "%s%06d.jpg", FileNamePrefix, FrameNum);
  
  OutFile = std::fopen(FileName, "w");
  if(!OutFile) ErrorEncountered = true;
}


void mxcpcJPEGSequenceWriter::endFrame(void) {

  if(OutFile) std::fclose(OutFile);
  
  OutFile = 0;
  
  FrameNum++;
}
  

void mxcpcJPEGSequenceWriter::receiveJPEGBytes(const mxcpc::u8 *buffer, 
                                               int num)                 {

  if(!OutFile) {
  
    ErrorEncountered = true;
    return;
  }
  
  if(num < 1) {
    
    ErrorEncountered = true;  
  
    return;
  }
  
  if(fwrite(buffer, num, 1, OutFile) != 1) {
    
    std::fclose(OutFile);
    OutFile = 0;
    
    ErrorEncountered = true;
  }
}
