//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcMxPEGRawFrameDumper.cpp,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2006/01/13 13:53:08 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcMxPEGRawFrameDumper.h>
#include <mxcpcByteStreamReceiver.h>

#include <memory>



mxcpcMxPEGRawFrameDumper
 ::mxcpcMxPEGRawFrameDumper(mxcpcByteStreamReceiver *frame_receiver,
                            bool convert_to_YUV)                     {

  FrameReceiver = frame_receiver;
  ConvertToYUV = convert_to_YUV;
  
  YUVBuffer = 0;
  
  ExplicitDumpMode = false;
}


mxcpcMxPEGRawFrameDumper::~mxcpcMxPEGRawFrameDumper() {

  delete FrameReceiver;
  
  delete[] YUVBuffer;
}



void mxcpcMxPEGRawFrameDumper::setExplicitDumpModeEnabled(bool enabled) {

  ExplicitDumpMode = enabled;
}


void mxcpcMxPEGRawFrameDumper::dumpFrame(void) { 

  int row_size;
  int x, y;
  mxcpc::u32 *line_start_ptr;
  mxcpc::u8 *ptr;
  int yuv_bytenum, yuv_uv_bytenum;
  int yuv_Cr_offset, yuv_Cb_offset;
  mxcpc::u8 col_y, col_Cb, col_Cr;
  int i;
  mxcpc::u8 *yuv_Cr_ptr, *yuv_Cb_ptr;
  
  if(!PixelBuffer) return;
  if(ConvertToYUV && !YUVBuffer) return;
  
  row_size = TileNumX*16*3/4;   // in u32 units
  
  line_start_ptr = PixelBuffer 
                    + PixelBufferOffset
                    + (Height - 1) * row_size;
  
  if(!ConvertToYUV) {
  
    for(y = 0; y < Height; y++) {
    
      ptr = (mxcpc::u8 *)line_start_ptr;
    
      FrameReceiver->receiveStreamBytes(ptr, Width * 3);
    
      line_start_ptr -= row_size;
    }
  }
  
  else {
    
    yuv_bytenum = Width*Height + 2 * Width*Height/4;   // Width, Height even
    yuv_uv_bytenum = 2 * Width*Height/4;
    yuv_Cr_offset = Width*Height;
    yuv_Cb_offset = yuv_Cr_offset + Width*Height/4;
    yuv_Cr_ptr = YUVBuffer + yuv_Cr_offset;
    yuv_Cb_ptr = YUVBuffer + yuv_Cb_offset;
    
    for(i = 0; i < yuv_uv_bytenum; i++) YUVBuffer[yuv_Cr_offset + i] = 0;
    
    for(y = 0; y < Height; y++) {
      
      ptr = (mxcpc::u8 *)line_start_ptr;
      for(x = 0; x < Width; x++) {
      
        mxcpc::convert_RGB_to_YCbCr(ptr[0], ptr[1], ptr[2],
                                    &col_y, &col_Cb, &col_Cr);
        ptr += 3;
        
        YUVBuffer[y*Width + x] =  col_y;
        *yuv_Cb_ptr            += (col_Cb>>2);
        *yuv_Cr_ptr            += (col_Cr>>2);

        if(x & 1) {
          yuv_Cb_ptr++;
          yuv_Cr_ptr++;
        }
      }
      
      line_start_ptr -= row_size;
      if(!(y & 1)) {
        yuv_Cb_ptr -= Width>>1;
        yuv_Cr_ptr -= Width>>1;
      }
    }
    
    FrameReceiver->receiveStreamBytes(YUVBuffer, yuv_bytenum);
  }
}


void mxcpcMxPEGRawFrameDumper::videoResolutionChanged(int width, int height) {

  char txt[100];
  
  mxcpcBufferedMxPEGDecoderBackEnd::videoResolutionChanged(width, height);
  
  delete[] YUVBuffer;
  YUVBuffer = 0;
  
  if(PixelBuffer) {
    
    if(width%2 || height%2) {
      mxcpc::sendStatusMsg("MxPEGRawFrameDumper : cannot allocate YUV buffer "
                           "for odd image dimensions!!!");
      errorEncountered();
    }
    
    try {
      
      YUVBuffer = new mxcpc::u8[width*height + 2 * width*height/4];
      
      std::sprintf(txt, "MxPEGRawFrameDumper : resized YUV buffer to %dx%d",
                   width, height);
      mxcpc::sendStatusMsg(txt);
      
    } catch(std::bad_alloc) {
            
        std::sprintf(txt, "MxPEGRawFrameDumper : failed to resize YUV buffer "
                          "to  %dx%d !!!",
                     width, height);
        mxcpc::sendStatusMsg(txt);
        
        errorEncountered();
        
        // YUVBuffer still 0
      }
  }
}


void mxcpcMxPEGRawFrameDumper::errorEncountered(void) {

}


void mxcpcMxPEGRawFrameDumper::frameComplete(void) {

  if(!ExplicitDumpMode) dumpFrame();
}


void mxcpcMxPEGRawFrameDumper::videoTileReceived(void) {

}

