//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcStreamFile.cpp,v $
//       ///       XXX       ///     $Revision: 1.5 $
//      ///      XXX XXX      ///     $Date: 2006/01/12 16:40:54 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcStreamFile.h>

#include <unistd.h>
#include <fcntl.h> 



mxcpcStreamFile::mxcpcStreamFile(const char *filename) {

  InFile = open(filename, O_RDONLY);
  
  if(InFile == -1) {
    CloseFileUponDestruction = false;
    StillUp = false;
  }
  else {
    CloseFileUponDestruction = true;
    StillUp = true;
  }
  
  TimedBlockMode = false;
  BlockTimeout   = 1000;
}


mxcpcStreamFile::mxcpcStreamFile(int fd) {

  InFile = fd;
  CloseFileUponDestruction = false;
  StillUp = true;
  
  TimedBlockMode = false;
  BlockTimeout   = 1000;
}


mxcpcStreamFile::~mxcpcStreamFile() {

  if(CloseFileUponDestruction) close(InFile);
}



void mxcpcStreamFile::setTimedBlockModeEnabled(bool enabled, long timeout) {

  TimedBlockMode = enabled;
  
  if(TimedBlockMode) BlockTimeout = timeout;
}


int mxcpcStreamFile::fetchBytes(unsigned char *buffer, int num) {

  int actually_read;
  struct timeval timeout;
  long usecs;
  fd_set file_descriptors;
  
  if(!StillUp) return(0);
  
  if(TimedBlockMode) {

    usecs = BlockTimeout * 1000;
    
    timeout.tv_sec  = usecs / 1000000;
    timeout.tv_usec = usecs % 1000000;
    
    FD_ZERO(&file_descriptors);
    FD_SET(InFile, &file_descriptors);
    if(!select(InFile + 1, &file_descriptors, 0, 0, &timeout)) return(0);
  }
  
  actually_read = read(InFile, buffer, num);
   
  if(actually_read <= 0) {
  
    StillUp = false;
    return(0);
  }
  
  return(actually_read);
}


bool mxcpcStreamFile::stillUp(void) {

  return(StillUp);
}

