//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcStreamFile_win.cpp,v $
//       ///       XXX       ///     $Revision: 1.5 $
//      ///      XXX XXX      ///     $Date: 2006/01/16 15:37:00 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcStreamFile.h>

#include <fcntl.h>
#include <windows.h>



/*! 
 *  \file mxcpcStreamFile_win.cpp
 *
 *  \brief Alternate implementation for the <tt>WINDOWS</tt> platform
 * 
 *  Differs from the original <tt>POSIX</tt>-dependent implementation in that 
 *  we explicitly have to request binary mode when doing an <tt>open()</tt> or
 *  <tt>fopen()</tt> to avoid having our carriage returns (13s) implicitly 
 *  toasted.
 */

 

mxcpcStreamFile::mxcpcStreamFile(const char *filename) {

  InFile = open(filename, O_RDONLY | O_BINARY);
  
  if(InFile == -1) {
    CloseFileUponDestruction = false;
    StillUp = false;
  }
  else {
    OSFHandle = _get_osfhandle(InFile);
    CloseFileUponDestruction = true;
    StillUp = true;
  }
  
  RestoreMode = -1;
  
  TimedBlockMode = false;
  BlockTimeout   = 1000;
}


mxcpcStreamFile::mxcpcStreamFile(int fd) {

  InFile = fd;
  
  if((RestoreMode = setmode(fd, O_BINARY)) != -1) { 
    OSFHandle = _get_osfhandle(InFile);
    StillUp = true;
  }
  else {
    StillUp = false;
  }
  
  CloseFileUponDestruction = false;
  
  TimedBlockMode = false;
  BlockTimeout   = 1000;
}


mxcpcStreamFile::~mxcpcStreamFile() {

  if(RestoreMode != -1) setmode(InFile, RestoreMode);
  
  if(CloseFileUponDestruction) close(InFile);
}



void mxcpcStreamFile::setTimedBlockModeEnabled(bool enabled, long timeout) {

  TimedBlockMode = enabled;
  
  if(TimedBlockMode) BlockTimeout = timeout;
}


int mxcpcStreamFile::fetchBytes(unsigned char *buffer, int num) {

  int actually_read;
  
  if(!StillUp) return(0);
  
  if(TimedBlockMode) {
    if(WaitForSingleObject((HANDLE)OSFHandle, BlockTimeout) 
        == WAIT_TIMEOUT) return(0);
  }
  
  actually_read = read(InFile, buffer, num);
   
  if(actually_read <= 0) {
  
    StillUp = false;
    return(0);
  }
  
  return(actually_read);
}


bool mxcpcStreamFile::stillUp(void) {

  return(StillUp);
}

