//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcSimpleMxPEGViewer.h,v $
//       ///       XXX       ///     $Revision: 1.7 $
//      ///      XXX XXX      ///     $Date: 2006/01/18 17:24:59 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_SIMPLEMXPEGVIEWER_H__
#define __MXCPC_SIMPLEMXPEGVIEWER_H__



#include <mxcpcVideoDisplay.h>



class mxcpcIPAddress;
class mxcpcProxyConfiguration;
class mxcpcMxPEGCodecFactory;
class mxcpcFramewiseMxPEGDecoder;
class mxcpcAsynchronousCameraLiveMxPEGFastStream;
class mxcpcBufferedMxPEGVideoRenderer;
class mxcpcAuthenticationDialog;
class mxcpcMessageDialog;

class QTimer;
class QMenu;



//! Simple, one-channel <tt>MxPEG</tt> viewer widget for early demo 
//! configurations.
/*!
 * \ingroup mxcpc_gui
 */
class mxcpcSimpleMxPEGViewer : public mxcpcVideoDisplay {
   
  Q_OBJECT
  
 private:
  mxcpcMxPEGCodecFactory *CodecFactory;
  mxcpcFramewiseMxPEGDecoder *Decoder;
  mxcpcAsynchronousCameraLiveMxPEGFastStream *FastStream;
  mxcpcBufferedMxPEGVideoRenderer *Renderer;  
  QTimer *ConnectionWatchdogTimer;
  int LastDrawnFrame;
  int ConnectionTimeout;
  QMenu *Menu;
  mxcpcAuthenticationDialog *AuthenticationDialog;
  mxcpcMessageDialog *MessageDialog;
  QString WindowTitlePrefix;
  int FramerateMeasureInterval;
  
 public:
  //! Well, constructor.
  mxcpcSimpleMxPEGViewer(
   QWidget *parent,
   mxcpcMxPEGCodecFactory *codec_factory,
   const QString& window_title_prefix = QString("SimpleMxPEGViewer"),
   int connection_timeout = 2000
  );
  ~mxcpcSimpleMxPEGViewer();
  
 signals:
  //! Emitted if framerate measure mode is enabled and a new framerate measure
  //! becomes available (roughly every <i>n</i> milliseconds with <i>n</i> 
  //! as it was specified to setFramerateMeasureModeEnabled()).
  void framerateMeasured(float fps);
  
 public slots:
  //! Connects to the camera specified and begins displaying the live stream.
  void playCameraLiveFastStream(
         const mxcpcIPAddress& camera_ip,
         int camera_port,
         const mxcpcProxyConfiguration& camera_proxy_configuration, 
         float fps);
  //! Disconnects any potential stream source asnd switches to disconnected
  //! state.
  void stop(void);

 public:
  //! Adds an item to the viewer's context menu that is raised by right-clicking
  //! into the display.
  void addMenuItem(const QString& item_text,
                   const QObject *receiver, const char *slot);
  //! Adds a horizontal separator to the viewer's popup menu.
  void addMenuSeparator(void);
  //! Toggles framerate measure mode.
  void setFramerateMeasureModeEnabled(bool enabled, 
                                      int measure_interval_ms = 10000);       
  
 protected:
  //! Reimplemented.
  void paintGL(void);
  //! Reimplemented.
  void resizeGL(int width, int height);

  //! Reimplemented.
  void mousePressEvent(QMouseEvent *e);
  
 private slots:
  void processIncoming(void);
  void processWatchdog(void);
  void processMeasuredFramerate(float fps);
};



#endif   // __MXCPC_SIMPLEMXPEGVIEWER_H__

