//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcSimpleMxPEGViewer.cpp,v $
//       ///       XXX       ///     $Revision: 1.10 $
//      ///      XXX XXX      ///     $Date: 2006/01/18 17:24:59 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcSimpleMxPEGViewer.h>
#include <mxcpcMxPEGCodecFactory.h>
#include <mxcpcAsynchronousCameraLiveMxPEGFastStream.h>
#include <mxcpcBufferedMxPEGVideoRenderer.h>
#include <mxcpcFramewiseMxPEGDecoder.h>
#include <mxcpcFramewiseMxPEGScanDecoderSoftwareOnly.h>
#include <mxcpcAuthenticationDialog.h>
#include <mxcpcMessageDialog.h>
#include <mxcpc_namespace.h>

#include <QTimer>
#include <QMenu>
#include <QMouseEvent>



/*!
 *  \param codec_factory
 *  will be used by the viewer when instantiating <tt>MxPEG</tt> decoding
 *  facilities. The viewer assumes ownership over the codec factory.
 *
 *  \param connection_timeout
 *  The timeout value specified will be added to the frame cycle time
 *  <tt>T = 1/framerate</tt> when engaging the stream connection watchdog
 *  timer.
 */
mxcpcSimpleMxPEGViewer
 ::mxcpcSimpleMxPEGViewer(QWidget *parent,
                          mxcpcMxPEGCodecFactory *codec_factory,
                          const QString& window_title_prefix,
                          int connection_timeout)
    : mxcpcVideoDisplay(parent)                               {

  WindowTitlePrefix = window_title_prefix;
  setWindowTitle(window_title_prefix);
  resize(640, 480);
  setAutoBufferSwap(false);
  
  CodecFactory = codec_factory;
  
  Decoder    = 0;   // viewer starts in disconnected state;
  FastStream = 0;
  Renderer   = 0;
  
  ConnectionWatchdogTimer = new QTimer(this);
  QObject::connect(ConnectionWatchdogTimer, SIGNAL(timeout(void)),
                   this, SLOT(processWatchdog(void)));
  LastDrawnFrame = 0;
  ConnectionTimeout = connection_timeout;
  if(ConnectionTimeout < 0) ConnectionTimeout = 2000;
  
  Menu = new QMenu(this);
  
  AuthenticationDialog = new mxcpcAuthenticationDialog();
  AuthenticationDialog->setWindowTitle(QString("Authentication"));
  MessageDialog = new mxcpcMessageDialog();

  FramerateMeasureInterval = -1;   // don't measure framerates
}
       

mxcpcSimpleMxPEGViewer::~mxcpcSimpleMxPEGViewer() {

  delete AuthenticationDialog;
  delete MessageDialog;
  
  delete Decoder;
  
  delete CodecFactory;
}



/*! 
 *  If the viewer was already streaming, the old stream source is disconected
 *  properly.
 */
void mxcpcSimpleMxPEGViewer
      ::playCameraLiveFastStream(
          const mxcpcIPAddress& camera_ip,
          int camera_port,
          const mxcpcProxyConfiguration& camera_proxy_configuration,
          float fps
        )                                                            {

  mxcpcAsynchronousCameraLiveMxPEGFastStream *fast_stream;
  QString window_title;
  int timeout;
  char txt[100];
  
  if(fps < 0.0f) fps = 4.0f;
  
  // disconnect a potentially established connection
  stop();
  
  // set up new decoding facility...
  FastStream 
   = fast_stream = new mxcpcAsynchronousCameraLiveMxPEGFastStream(
                         camera_ip,
                         camera_port,
                         camera_proxy_configuration,
                         fps
                       );
  Renderer = new mxcpcBufferedMxPEGVideoRenderer(this);
  Decoder = new mxcpcFramewiseMxPEGDecoder(
                 fast_stream,
                 Renderer,
                 CodecFactory->newFramewiseMxPEGScanDecoder()
                );
                
  QObject::connect(fast_stream, SIGNAL(bytesAvailable(void)),
                   this, SLOT(processIncoming(void)));
  QObject::connect(fast_stream, SIGNAL(authenticationRequest(QString)),
                   AuthenticationDialog, SLOT(openDialog(const QString&)));
  QObject::connect(AuthenticationDialog, 
                   SIGNAL(authenticationCommitted(QString, QString)),
                   fast_stream,
                   SLOT(processAuthentication(const QString&, const QString&)));
  QObject::connect(AuthenticationDialog, 
                   SIGNAL(authenticationCancelled(void)),
                   fast_stream,
                   SLOT(processAuthenticationCancellation(void)));
  QObject::connect(Renderer, SIGNAL(framerateMeasured(float)),
                   this, SLOT(processMeasuredFramerate(float)));
                   
  window_title = WindowTitlePrefix 
                  + QString(" - Camera ")
                  + QString::number(camera_ip.getIP1()) + QString(".")
                  + QString::number(camera_ip.getIP2()) + QString(".")
                  + QString::number(camera_ip.getIP3()) + QString(".")
                  + QString::number(camera_ip.getIP4())
                  + QString(" @ ");
  if(fps != 0.0f) {
    if(fps < 1.0f) std::sprintf(txt, "%.2f", fps);
    else           std::sprintf(txt, "%.0f", fps);
    window_title += QString(txt) + QString("fps");
  }
  else { 
    window_title += QString("Maximum Framerate");
  }
  setWindowTitle(window_title);
  
  LastDrawnFrame = Renderer->getFrameCount();
  if(fps != 0.0f)
   timeout = (int)((float)ConnectionTimeout + 1000.0f/fps);
  else
   timeout = ConnectionTimeout;
  ConnectionWatchdogTimer->start(timeout);
  
  MessageDialog->hide();
  
  if(FramerateMeasureInterval >= 0)
   Renderer->setFramerateMeasureModeEnabled(true, FramerateMeasureInterval);
  else
   Renderer->setFramerateMeasureModeEnabled(false);
}


/*!
 *  If the viewer is currently not connected to any stream source, the method
 *  has no effect.
 */
void mxcpcSimpleMxPEGViewer::stop(void) {
  
  // disconnect a potentially established connection
  delete Decoder;   // signal wiring to and from the authentication dialog
                    // will implicitly be disconnected
  Decoder    = 0;
  FastStream = 0;
  Renderer   = 0;
  ConnectionWatchdogTimer->stop();
  
  setWindowTitle(WindowTitlePrefix);
  updateGL();
}


void mxcpcSimpleMxPEGViewer::addMenuItem(const QString& item_text,
                                         const QObject *receiver, 
                                         const char *slot)        {

  Menu->addAction(item_text, receiver, slot);
}


void mxcpcSimpleMxPEGViewer::addMenuSeparator(void) {

  Menu->addSeparator();
}


/*!
 *   Per default, the framerate measure mode is disabled.
 */
void mxcpcSimpleMxPEGViewer
      ::setFramerateMeasureModeEnabled(bool enabled, int measure_interval_ms) {

  if(!enabled) FramerateMeasureInterval = -1;
  else {
    FramerateMeasureInterval = measure_interval_ms;
    if(FramerateMeasureInterval < 0) FramerateMeasureInterval = 10000;   // 10s
   
  }            
  
  if(Renderer) {
    if(FramerateMeasureInterval >= 0)
     Renderer->setFramerateMeasureModeEnabled(true, FramerateMeasureInterval);
    else
     Renderer->setFramerateMeasureModeEnabled(false); 
  }
}


void mxcpcSimpleMxPEGViewer::paintGL(void) {
 
  //if(Decoder && !AuthenticationDialog->isVisible()) return;
 
  glClearColor(0.0f,
               0.0f,
               0.5f,
               0.0f);
  glClear(GL_COLOR_BUFFER_BIT);
 
  // restore GL base state...
  glClearColor(0.0f,
               0.0f,
               0.0f,
               0.0f);
}


void mxcpcSimpleMxPEGViewer::resizeGL(int width, int height) {
        
  glViewport(0, 0, width, height);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho((GLdouble)0.0, (GLdouble)width,
          (GLdouble)height, (GLdouble)0.0,
          (GLdouble)-1.0, (GLdouble)1.0);
  glMatrixMode(GL_MODELVIEW);
}


void mxcpcSimpleMxPEGViewer::mousePressEvent(QMouseEvent *e) {
  
  if(e->button() == Qt::RightButton) {
    
    Menu->popup(e->globalPos());
  }
}


void mxcpcSimpleMxPEGViewer::processIncoming(void) {

  if(!Decoder) return;   // just be safe
  
  while(Decoder->decode(2048))
   ;
}


void mxcpcSimpleMxPEGViewer::processWatchdog(void) {

  int frame_count;
  
  if(!Decoder) return;   // just be safe
  
  frame_count = Renderer->getFrameCount();
  if((frame_count == LastDrawnFrame)
     && !FastStream->wasAuthenticationDialogRaised()) {
    MessageDialog->setMessage(QString("Notification"),
                              QString("Stream connection timed out!"));
    MessageDialog->openDialog();
    mxcpc::sendStatusMsg("mxcpcSimpleMxPEGViewer : "
                         "stream connection timed out - disconnecting...");
    stop();
  }
  else
   LastDrawnFrame = frame_count;
}


void mxcpcSimpleMxPEGViewer::processMeasuredFramerate(float fps) {
  
  char txt[100];
  
  std::sprintf(txt, "SimpleMxPEGViewer : current effective framerate : %.2f",
               fps);
  mxcpc::sendStatusMsg(txt);
  
  emit framerateMeasured(fps);
}

