//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcAsynchronousCameraLiveMxPEGFastStream.h,v $
//       ///       XXX       ///     $Revision: 1.5 $
//      ///      XXX XXX      ///     $Date: 2006/01/05 11:56:09 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#ifndef __MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_H__
#define __MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_H__



#define MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_HEADERBUFFER_SIZE 1024



#include <mxcpcCameraLiveMxPEGFastStream.h>
#include <mxcpcIPAddress.h>
#include <mxcpcProxyConfiguration.h>

#include <QTcpSocket>



//! Camera FastStream implementation using an asynchronous <tt>Qt</tt>
//! TCP socket.
/*!
    \ingroup mxcpc_tek
    
    Due to the socket's asynchronous nature the fetchBytes() method will not 
    block in case no data is available - it will return <tt>0</tt> instead. To
    circumvent the need for polling, connect to the bytesAvailable() signal that
    will fire when new data comes in over the network.

    If on the specified camera access to live streaming is restricted, the
    stream source will automatically handle the HTTP negotiation necessary to
    properly authenticate the user, including requesting and authentification 
    information via  appropriate signals and slots. Failure to authenticate is
    logged via the <tt>mxcpc</tt> status message facility.
*/
class mxcpcAsynchronousCameraLiveMxPEGFastStream
       : public QObject,
         public mxcpcCameraLiveMxPEGFastStream   {

  Q_OBJECT

 private:
  enum HTTPNegotiationState {
    SendGET_NoAuth,
    ReceiveResponseHeader_NoAuth,
    RaisedAuthenticationDialog,
    WaitForAuthenticationDialog,
    SendGET_EnteredAuth,
    ReceiveResponseHeader_EnteredAuth,
    NegotiationSucceeded,
    NegotiationFailed
  };
  
 private:
  QTcpSocket *AsyncSocket;
  mxcpcIPAddress CameraIP;
  int CameraPort;
  mxcpcProxyConfiguration CameraProxyConfiguration;
  float FPS;
  bool HTTPNegotiationMode;
  HTTPNegotiationState NegotiationState;
  char HTTPHeaderBuffer
        [MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_HEADERBUFFER_SIZE];
  int HTTPHeaderBytesRead;
  int HTTPHeaderSize;      
  bool AuthenticationEntered;
  bool AuthenticationDialogRaised;
  bool AuthenticationDialogWasRaised;
  QString AuthenticationUser;
  QString AuthenticationPassword;
  
 public:
  mxcpcAsynchronousCameraLiveMxPEGFastStream(
    const mxcpcIPAddress& camera_ip, 
    int camera_port,
    const mxcpcProxyConfiguration& camera_proxy_configuration,
    float fps = 4
  );
  mxcpcAsynchronousCameraLiveMxPEGFastStream(const mxcpcIPAddress& camera_ip, 
                                             int port,
                                             float fps = 4);
  mxcpcAsynchronousCameraLiveMxPEGFastStream(const mxcpcIPAddress& camera_ip, 
                                             int port,
                                             int fps = 4);
  mxcpcAsynchronousCameraLiveMxPEGFastStream(int ip1, int ip2, int ip3, int ip4, 
                                             int port,
                                             int fps = 4);
  ~mxcpcAsynchronousCameraLiveMxPEGFastStream();
 private:
  void constructCommonStuff(const mxcpcIPAddress& camera_ip, int camera_port,
                            float fps);  
  
 signals:
  //! Emitted when bytes have become available to be read.
  void bytesAvailable(void);
  //! Emitted when during negotiation with the camera, need arises to 
  //! get authentication information about the user.
  /*!
   *  Originally indended to be connected to an mycpcAuthenticationDialog's
   *  raise() slot.
   *
   *  By using the component-oriented signal/slot-mechanism, we avoid doing an
   *  upcall from the <tt>mxcpc_tek</tt> to the <tt>mxcpc_gui</tt> library.
   */
  void authenticationRequest(QString msg);
  
 public slots:
  //! To be activated from an external place after the faststream source has
  //! emmitted the authenticationRequest() signal, or, alternatively:
  //! the processAuthenticationCancellation() slot.
  void processAuthentication(const QString& user, const QString& password);
  //! To be activated from an external place after the faststream source has
  //! emmitted the authenticationRequest() signal, or, alternatively:
  //! the processAuthentication() slot.
  void processAuthenticationCancellation(void);
  
 public:
  //! Reimplemented.
  int fetchBytes(unsigned char *buffer, int num);
  //! Reimplemented.
  bool stillUp(void);
  //! Blocks for incoming.
  bool waitForBytes(int timeout_msecs = -1);
  //! Tells whether or not the stream source's authentication dialog is 
  //! currently raised.
  bool isAuthenticationDialogRaised(void);
  //! Tells whether or not the stream source's authentication dialog was 
  //! raised since the last call to this method.
  bool wasAuthenticationDialogRaised(void);
  
 private slots:
  //! Connected to the encapsuled socket's readyRead() signal.
  void processIncomingBytes(void);
  
 private:
  //! Internal method.
  void establishConnection(void);
  //! Internal method.
  void sendGETRequest(const char *user = 0, const char *password = 0);
  //! Internal method.
  bool receiveHTTPHeader(void);
};



#endif   // __MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_H__

