//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcAsynchronousCameraLiveMxPEGFastStream.cpp,v $
//       ///       XXX       ///     $Revision: 1.6 $
//      ///      XXX XXX      ///     $Date: 2006/01/05 11:56:09 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcAsynchronousCameraLiveMxPEGFastStream.h>
#include <mxcpc_namespace.h>

#include <cstdio>



mxcpcAsynchronousCameraLiveMxPEGFastStream
 ::mxcpcAsynchronousCameraLiveMxPEGFastStream(
     const mxcpcIPAddress& camera_ip, 
     int camera_port,
     const mxcpcProxyConfiguration& camera_proxy_configuration,
     float fps
   ) : mxcpcCameraLiveMxPEGFastStream(camera_ip.getIP1(),
                                      camera_ip.getIP2(),
                                      camera_ip.getIP3(),
                                      camera_ip.getIP4(),
                                      camera_port,
                                      (int)fps)                                {

  CameraProxyConfiguration = camera_proxy_configuration;
  
  constructCommonStuff(camera_ip,
                       camera_port,
                       fps);
}
                                              

mxcpcAsynchronousCameraLiveMxPEGFastStream
 ::mxcpcAsynchronousCameraLiveMxPEGFastStream(const mxcpcIPAddress& camera_ip,
                                              int port,
                                              float fps)
    : mxcpcCameraLiveMxPEGFastStream(camera_ip.getIP1(),
                                     camera_ip.getIP2(),
                                     camera_ip.getIP3(),
                                     camera_ip.getIP4(),
                                     port,
                                     (int)fps)                                 {

  // camera proxy config implicitly defaults to "no proxy"
                                       
  constructCommonStuff(camera_ip,
                       port,
                       fps);
}


mxcpcAsynchronousCameraLiveMxPEGFastStream
 ::mxcpcAsynchronousCameraLiveMxPEGFastStream(const mxcpcIPAddress& camera_ip,
                                              int port,
                                              int fps)
    : mxcpcCameraLiveMxPEGFastStream(camera_ip.getIP1(),
                                     camera_ip.getIP2(),
                                     camera_ip.getIP3(),
                                     camera_ip.getIP4(),
                                     port,
                                     fps)                                      {

  // camera proxy config implicitly defaults to "no proxy"
  
  constructCommonStuff(camera_ip,
                       port,
                       (float)fps);
}


mxcpcAsynchronousCameraLiveMxPEGFastStream
 ::mxcpcAsynchronousCameraLiveMxPEGFastStream(int ip1,
                                              int ip2, 
                                              int ip3, 
                                              int ip4, 
                                              int port,
                                              int fps)
    : mxcpcCameraLiveMxPEGFastStream(ip1,
                                     ip2,
                                     ip3,
                                     ip4,
                                     port,
                                     fps)               {

  // camera proxy config implicitly defaults to "no proxy"
  
  constructCommonStuff(mxcpcIPAddress(ip1, ip1, ip3, ip4),
                       port,
                       (float)fps);
}
    
                                   
void mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::constructCommonStuff(const mxcpcIPAddress& camera_ip, int camera_port,
                             float fps)                                        {
  
  CameraIP   = camera_ip;
  CameraPort = camera_port;
  FPS        = fps;
  if(FPS < 0.0f) FPS = 4.0f;
  
  mxcpc::sendStatusMsg((
          QString("AsynchronousCameraLiveMxPEGFastStream :")
          + QString(" opening faststream connection to camera ")
          + QString::number(CameraIP.getIP1())
          + QString(".")
          + QString::number(CameraIP.getIP2())
          + QString(".")
          + QString::number(CameraIP.getIP3())
          + QString(".")
          + QString::number(CameraIP.getIP4())
          + QString("...")
         ).toLatin1().constData());
  
  AsyncSocket = new QTcpSocket(0);
  QObject::connect(AsyncSocket, SIGNAL(connected(void)),
                   this, SLOT(processIncomingBytes(void)));
  QObject::connect(AsyncSocket, SIGNAL(readyRead(void)),
                   this, SLOT(processIncomingBytes(void)));                 
  
  establishConnection();
    
  HTTPHeaderBytesRead           = 0;
  HTTPHeaderSize                = 0;
  AuthenticationEntered         = false;
  AuthenticationDialogRaised    = false;
  AuthenticationDialogWasRaised = false;
  AuthenticationUser            = QString("");
  AuthenticationPassword        = QString("");
  
  NegotiationState              = SendGET_NoAuth;
  HTTPNegotiationMode           = true;
  // negotiation will be initiated by the socket's connected() signal 
  // activating our processIncomingBytes() slot
}


mxcpcAsynchronousCameraLiveMxPEGFastStream
 ::~mxcpcAsynchronousCameraLiveMxPEGFastStream() {
         
  mxcpc::sendStatusMsg((
          QString("AsynchronousCameraLiveMxPEGFastStream :")
          + QString(" closing faststream connection to camera ")
          + QString::number(CameraIP.getIP1())
          + QString(".")
          + QString::number(CameraIP.getIP2())
          + QString(".")
          + QString::number(CameraIP.getIP3())
          + QString(".")
          + QString::number(CameraIP.getIP4())
          + QString(".")
         ).toLatin1().constData()); 
   
  delete AsyncSocket;
}



int mxcpcAsynchronousCameraLiveMxPEGFastStream
     ::fetchBytes(unsigned char *buffer, int num) {
  
  int result;
  
  result = AsyncSocket->read((char *)buffer, num);
  if(result < 0) {
   
    // ERROR CONDITION !!! PROCESS MORE THOROUGHLY!!!
    result = 0;
  }
  
  return(result);
}


bool mxcpcAsynchronousCameraLiveMxPEGFastStream::stillUp(void) {

  std::printf("*** AsynchronousCameraLiveMxPEGFastStream::stillUp() "
              "not yet implemented !!! ***");
  std::exit(666);
}


/*!
 *  \param timeout_msecs
 *  If not <tt>-1</tt> the wait operation will time out after the number of
 *  milliseconds specified.
 *
 *  \returns
 *  <tt>true</tt> if data is available, and <tt>false</tt> in case of error or
 *  timeout
 */
bool mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::waitForBytes(int timeout_msecs)         {
   
  return(AsyncSocket->waitForReadyRead(timeout_msecs));
  
}


bool mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::isAuthenticationDialogRaised(void)    {
        
  return(AuthenticationDialogRaised);
}


/*!
 *  If the dialog is currently raised the method will return <tt>true</tt> and
 *  the internal raise flag will not be reset.
 */
bool mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::wasAuthenticationDialogRaised(void)    {
        
  bool result;
        
  result = AuthenticationDialogWasRaised;
  
  if(!AuthenticationDialogRaised) AuthenticationDialogWasRaised = false;
  
  return(result);
}


void mxcpcAsynchronousCameraLiveMxPEGFastStream::processIncomingBytes(void) {
  
  int first_carriage_return;
  
  if(HTTPNegotiationMode) {
    
    for(;;) {

      switch(NegotiationState) {
      
       case SendGET_NoAuth:
         sendGETRequest();
         NegotiationState = ReceiveResponseHeader_NoAuth;
         break;

       case ReceiveResponseHeader_NoAuth:
         if(!receiveHTTPHeader()) return;
         
         first_carriage_return = 0;
         while(   (HTTPHeaderBuffer[first_carriage_return] != '\r')
               && (HTTPHeaderBuffer[first_carriage_return] != '\n'))  // be safe
           first_carriage_return++;
         if((first_carriage_return >= 2)
            && !std::strncmp(&HTTPHeaderBuffer[first_carriage_return - 2],
                             "OK", 2))                                     {   
           NegotiationState = NegotiationSucceeded;
         }
         else if((first_carriage_return >= 12)
                 && !std::strncmp(&HTTPHeaderBuffer[first_carriage_return - 12],
                                  "Unauthorized", 12))                         {
           AsyncSocket->disconnectFromHost();
           emit authenticationRequest(QString("Camera <b>")
                                      + QString::number(CameraIP.getIP1())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP2())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP3())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP4())
                                      + QString("</b> ")
                                      + QString("requires you to authenticate ")
                                      + QString("for live stream access."));
           NegotiationState = RaisedAuthenticationDialog;
         }
         else {
           NegotiationState = NegotiationFailed;
         }
         break;
         
       case RaisedAuthenticationDialog:
         AuthenticationEntered = false;
         AuthenticationDialogRaised = true;
         AuthenticationDialogWasRaised = true;
         NegotiationState = WaitForAuthenticationDialog;
         break;
  
       case WaitForAuthenticationDialog:
         if(!AuthenticationEntered) return;
         // AuthenticationDialogRaised is false again.
         establishConnection();
         NegotiationState = SendGET_EnteredAuth;
         return;   // SendGET_EnteredAuth will be reached by the socket's
                   // conected() signal activating us (processIncomingBytes())
                   // again
         
       case SendGET_EnteredAuth:
         sendGETRequest(AuthenticationUser.toLatin1().constData(),
                        AuthenticationPassword.toLatin1().constData());
         NegotiationState = ReceiveResponseHeader_EnteredAuth;
         break;
         
       case ReceiveResponseHeader_EnteredAuth:
         if(!receiveHTTPHeader()) return;
         
         first_carriage_return = 0;
         while(   (HTTPHeaderBuffer[first_carriage_return] != '\r')
               && (HTTPHeaderBuffer[first_carriage_return] != '\n'))  // be safe
           first_carriage_return++;
         if((first_carriage_return >= 2)
            && !std::strncmp(&HTTPHeaderBuffer[first_carriage_return - 2],
                             "OK", 2))                                         {
           mxcpc::sendStatusMsg((
                   QString("AsynchronousCameraLiveMxPEGFastStream :")
                   + QString(" user '")
                   + AuthenticationUser
                   + QString("' successfully authenticated for live stream ")
                   + QString("access to camera ")
                   + QString::number(CameraIP.getIP1())
                   + QString(".")
                   + QString::number(CameraIP.getIP2())
                   + QString(".")
                   + QString::number(CameraIP.getIP3())
                   + QString(".")
                   + QString::number(CameraIP.getIP4())
                   + QString(".")
                  ).toLatin1().constData());
           NegotiationState = NegotiationSucceeded;
         }
         else if((first_carriage_return >= 12)
                 && !std::strncmp(&HTTPHeaderBuffer[first_carriage_return - 12],
                                  "Unauthorized", 12))                         {
           mxcpc::sendStatusMsg((
                   QString("AsynchronousCameraLiveMxPEGFastStream :")
                   + QString(" *** user '")
                   + AuthenticationUser
                   + QString("' failed to authenticate for live stream ")
                   + QString("access to camera ")
                   + QString::number(CameraIP.getIP1())
                   + QString(".")
                   + QString::number(CameraIP.getIP2())
                   + QString(".")
                   + QString::number(CameraIP.getIP3())
                   + QString(".")
                   + QString::number(CameraIP.getIP4())
                   + QString(" ***")
                  ).toLatin1().constData());
           emit authenticationRequest(QString("Camera <b>")
                                      + QString::number(CameraIP.getIP1())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP2())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP3())
                                      + QString(".")
                                      + QString::number(CameraIP.getIP4())
                                      + QString("</b> ")
                                      + QString("rejected your authentication ")
                                      + QString("request. Please try again."));
           NegotiationState = RaisedAuthenticationDialog;
         }
         else {
           NegotiationState = NegotiationFailed;
         }
         break;
         
       case NegotiationSucceeded:
         mxcpc::sendStatusMsg((
                 QString("AsynchronousCameraLiveMxPEGFastStream :")
                 + QString(" receiving faststream from camera ")
                 + QString::number(CameraIP.getIP1())
                 + QString(".")
                 + QString::number(CameraIP.getIP2())
                 + QString(".")
                 + QString::number(CameraIP.getIP3())
                 + QString(".")
                 + QString::number(CameraIP.getIP4())
                 + QString(".")
                ).toLatin1().constData());
         HTTPNegotiationMode = false;
         if(AsyncSocket->bytesAvailable()) emit bytesAvailable();
         return;
         
       case NegotiationFailed:
       default:
         return;
      }
    }
  }
  
  else {
   
    emit bytesAvailable();
  }
}


void mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::processAuthentication(const QString& user, const QString& password) {

  AuthenticationUser         = user;
  AuthenticationPassword     = password;
  
  AuthenticationDialogRaised = false;
  AuthenticationEntered      = true;
  
  processIncomingBytes();
}


void mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::processAuthenticationCancellation(void) {
  
  AuthenticationDialogRaised = false;
}


void mxcpcAsynchronousCameraLiveMxPEGFastStream::establishConnection(void) {

  char txt[100];
  
  if(CameraProxyConfiguration.proxyUsed()) {
    std::sprintf(txt, "%d.%d.%d.%d", 
                 CameraProxyConfiguration.serverIP().getIP1(),
                 CameraProxyConfiguration.serverIP().getIP2(),
                 CameraProxyConfiguration.serverIP().getIP3(),
                 CameraProxyConfiguration.serverIP().getIP4());
                                     
    AsyncSocket->connectToHost(QString(txt), 
                               CameraProxyConfiguration.serverPort());
  }
  else {
    std::sprintf(txt, "%d.%d.%d.%d", CameraIP.getIP1(),
                                     CameraIP.getIP2(),
                                     CameraIP.getIP3(),
                                     CameraIP.getIP4());
    AsyncSocket->connectToHost(QString(txt), CameraPort);
  }
}


void mxcpcAsynchronousCameraLiveMxPEGFastStream
      ::sendGETRequest(const char *user, 
                       const char *password) {

  char send_buffer[1024];
  char txt_buffer[110], big_txt_buffer[200];
  
  // compose HTTP GET request... 
  if(!CameraProxyConfiguration.proxyUsed()) {
    std::sprintf(send_buffer, 
                 "GET /control/faststream.jpg?noaudio&fps=%f HTTP/1.0\r\n",
                 FPS);
  }
  else {
    std::sprintf(send_buffer, 
                 "GET http://%d.%d.%d.%d:%d/control/faststream.jpg"
                 "?noaudio&fps=%f HTTP/1.0\r\n",
                 CameraIP.getIP1(),
                 CameraIP.getIP2(),
                 CameraIP.getIP3(),
                 CameraIP.getIP4(),
                 CameraPort,
                 FPS);
  }
  if(user && password && (std::strlen(user) + std::strlen(password) < 100)) {
    std::sprintf(txt_buffer, "%s:%s", user, password);
    mxcpc::encode_base64((unsigned char *)txt_buffer, std::strlen(txt_buffer),
                         (unsigned char *)big_txt_buffer);
    std::strcat(send_buffer, "Authorization: BASIC ");
    std::strcat(send_buffer, big_txt_buffer);
    std::strcat(send_buffer, "\r\n");         
  }
  if(CameraProxyConfiguration.proxyUsed()) {
    std::strcat(send_buffer, "Host: ");
    std::sprintf(big_txt_buffer, "%d.%d.%d.%d:%d",
                 CameraIP.getIP1(),
                 CameraIP.getIP2(),
                 CameraIP.getIP3(),
                 CameraIP.getIP4(),
                 CameraPort);
    std::strcat(send_buffer, big_txt_buffer);
    std::strcat(send_buffer, "\r\n");      
  }
  std::strcat(send_buffer, "\r\n");              
  
  // send GET request...
  AsyncSocket->write(send_buffer, std::strlen(send_buffer));
    // We don't do any error checking here. Errors will be detected via the
    // stream connection timeout detection done by the superior context... 
}   


/*!
 *  \returns <tt>true</tt> if the complete header has successfully been read 
 *           into the buffer, or <tt>false</tt> if header bytes are still
 *           missing. 
 */
bool mxcpcAsynchronousCameraLiveMxPEGFastStream::receiveHTTPHeader(void) {

  char c;
  
  if(HTTPHeaderBytesRead 
      == MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_HEADERBUFFER_SIZE)
    return(false);
      // buffer is full - "dead end" will be resolved by timeout strategy of
      // superior context
  
  while(AsyncSocket->read(&c, 1) == 1) {
    
    HTTPHeaderBuffer[HTTPHeaderBytesRead] = c;
    HTTPHeaderBytesRead++;
    
    if(   ((HTTPHeaderBytesRead >= 2) 
           && !std::strncmp(&HTTPHeaderBuffer[HTTPHeaderBytesRead - 2], 
                            "\n\n", 2))   // old camera software does it wrong
       || ((HTTPHeaderBytesRead >= 4) 
           && !std::strncmp(&HTTPHeaderBuffer[HTTPHeaderBytesRead - 4], 
                            "\r\n\r\n", 4)))                            {
                            
      HTTPHeaderSize = HTTPHeaderBytesRead;
      HTTPHeaderBytesRead = 0;
      return(true);
    }
    else {
      if(HTTPHeaderBytesRead 
          == MXCPC_ASYNCHRONOUSCAMERALIVEMXPEGFASTSTREAM_HEADERBUFFER_SIZE)
      return(false);
    }
  }
  
  return(false);
}

