//           ///          //                                        Mx clientSDK
//          /////        ////                    Mx Crossplatform Client Library
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcBufferedMxPEGVideoRenderer.cpp,v $
//       ///       XXX       ///     $Revision: 1.4 $
//      ///      XXX XXX      ///     $Date: 2006/01/18 17:24:59 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems //////////////////////////////////////////////
//                                                                          //
//  Copyright (C) 2005 - 2006, MOBOTIX AG, Germany                         //
//  This software is made available under the BSD licence. Please refer   //
//  to the file LICENCE.TXT contained in this distribution for details.  //
//                                                                      //
//  /////////////////////////////////////////////////////////////////////



#include <mxcpcBufferedMxPEGVideoRenderer.h>
#include <mxcpcVideoDisplay.h>



mxcpcBufferedMxPEGVideoRenderer
 ::mxcpcBufferedMxPEGVideoRenderer(mxcpcVideoDisplay *viewer) {
  
  Viewer = viewer;
  
  CenterX = CenterY = 0;
  OffsetX = OffsetY = 0;
  
  FrameCount = 0;
  
  FramerateMeasureInterval = -1;   // don't measure framerate
  FramerateMeasureTime.start();
  FramerateFrameCount = 0;
}


mxcpcBufferedMxPEGVideoRenderer::~mxcpcBufferedMxPEGVideoRenderer() {

}


/*!
 *   Per default, the framerate measure mode is disabled.
 */
void mxcpcBufferedMxPEGVideoRenderer
      ::setFramerateMeasureModeEnabled(bool enabled, int measure_interval_ms) {
  
  if(!enabled) FramerateMeasureInterval = -1;
  else {
    FramerateMeasureInterval = measure_interval_ms;
    if(FramerateMeasureInterval < 0) FramerateMeasureInterval = 10000;   // 10s
    FramerateMeasureTime.start();
    FramerateFrameCount = 0;
  }
}


int mxcpcBufferedMxPEGVideoRenderer::getFrameCount(void) {

  return(FrameCount);
}


void mxcpcBufferedMxPEGVideoRenderer::videoResolutionChanged(int width, 
                                                             int height) {
  
  mxcpcBufferedMxPEGDecoderBackEnd::videoResolutionChanged(width, height);
  
  if(PixelBuffer) {
    CenterX = Width/2;    OffsetX = Width  - CenterX;
    CenterY = Height/2;   OffsetY = Height - CenterY;
  }
  else {
    CenterX = CenterY = 0;
    OffsetX = OffsetY = 0;
  }
  
  Viewer->updateGL();
}


void mxcpcBufferedMxPEGVideoRenderer::errorEncountered(void) {

}


void mxcpcBufferedMxPEGVideoRenderer::frameComplete(void) {

  int WinWidth, WinHeight;   // letter-case convention violation, dammit...
  int elapsed;
  float fps;
  
  // measure framerate, if enabled...
  fps = -1.0f;
  if(FramerateMeasureInterval >= 0) {
    
    FramerateFrameCount++;
    
    if((elapsed = FramerateMeasureTime.elapsed()) >= FramerateMeasureInterval) {
      
      fps = (float)FramerateFrameCount * 1000.0f / (float)elapsed;
      
      FramerateMeasureTime.start();
      FramerateFrameCount = 0;
    }
  }
  
  if(!PixelBuffer) {
  
    if(fps >= 0.0f) emit framerateMeasured(fps);
    
    return;
  }
  
  WinWidth  = Viewer->width();
  WinHeight = Viewer->height(); 
  
  Viewer->makeCurrent();
  
  if(   false   // scale to fit...   (currently disabled)
     && (   ((WinWidth > Width ? WinWidth - Width : Width - WinWidth) > 2)
         || ((WinHeight > Height ? WinHeight - Height : Height - WinHeight) > 2)
        )
    )                                                                          {
          
    glRasterPos2i(WinWidth/2, WinHeight/2);
    glBitmap(0,                        0, 
             0.0f,                     0.0f,
             (GLfloat)(-(WinWidth/2)), (GLfloat)(-(WinHeight/2 + 1)),
             0);
    glPixelZoom((GLfloat)WinWidth / (GLfloat)Width,
                (GLfloat)WinHeight / (GLfloat)Height);
    glDrawPixels(Width, Height,
                 GL_RGB, GL_UNSIGNED_BYTE,
                 PixelBuffer + PixelBufferOffset);
    glPixelZoom(1.0f, 1.0f);
  }
  
  else {   // center...
                  
    glRasterPos2i(WinWidth/2, WinHeight/2);
    glBitmap(0,                  0, 
             0.0f,               0.0f,
             (GLfloat)-CenterX, (GLfloat)-CenterY,
             0);
    glDrawPixels(Width, Height,
                 GL_RGB, GL_UNSIGNED_BYTE,
                 PixelBuffer + PixelBufferOffset);
  }
  
  glFlush();
  
  FrameCount++;
  if(FrameCount < 0) FrameCount = 0;   // overflow?
  
  if(fps >= 0.0f) emit framerateMeasured(fps);
}


void mxcpcBufferedMxPEGVideoRenderer::videoTileReceived(void) {

}

