//           ///          //
//          /////        ////
//         /// XXX     XXX ///
//        ///    XXX XXX    ///     $RCSfile: mxcpcFramewiseMxPEGScanDecoderIPP.cpp,v $  
//       ///       XXX       ///     $Revision: 1.2 $
//      ///      XXX XXX      ///     $Date: 2005/12/20 18:04:19 $
//     ////    XXX     XXX    ////     $Author: cvs-kai $
//    ////                     ////
//   ////  M  O  B  O  T  I  X  ////////////////////////////////////////////////
//  //// Security Vision Systems ///////////////////////////////////////////////



#include <mxcpcFramewiseMxPEGScanDecoderIPP.h>
#include <mxcpcMxPEGDecoderBackEnd.h>
#include <mxcpc_exceptions.h>

#include <ippcc.h>

#include <exception>
#include <memory>


                        
mxcpcFramewiseMxPEGScanDecoderIPP::mxcpcFramewiseMxPEGScanDecoderIPP() {

  int huffman_size;
  int i;
  
  ippiDecodeHuffmanSpecGetBufSize_JPEG_8u(&huffman_size);
  
  HuffmanTree_Y_DC  = 0;
  HuffmanTree_Y_AC  = 0;
  HuffmanTree_UV_DC = 0;
  HuffmanTree_UV_AC = 0;
  
  HuffmanTree_Y_DC  = (IppiDecodeHuffmanSpec *)std::malloc(huffman_size);
  HuffmanTree_Y_AC  = (IppiDecodeHuffmanSpec *)std::malloc(huffman_size);
  HuffmanTree_UV_DC = (IppiDecodeHuffmanSpec *)std::malloc(huffman_size);
  HuffmanTree_UV_AC = (IppiDecodeHuffmanSpec *)std::malloc(huffman_size);
  
  if(   !HuffmanTree_Y_DC
     || !HuffmanTree_Y_AC
     || !HuffmanTree_UV_DC
     || !HuffmanTree_UV_AC) {
    
    std::free(HuffmanTree_Y_DC);
    std::free(HuffmanTree_Y_AC);
    std::free(HuffmanTree_UV_DC);
    std::free(HuffmanTree_UV_AC);
    throw std::bad_alloc();
  }
  
  try {
     
    if(ippiDecodeHuffmanSpecInit_JPEG_8u(&DefaultHuffmanTable_Y_DC[0],
                                         &DefaultHuffmanTable_Y_DC[16],
                                         HuffmanTree_Y_DC)
        != ippStsNoErr) throw mxcpcDidntWorkOutException();
  
    if(ippiDecodeHuffmanSpecInit_JPEG_8u(&DefaultHuffmanTable_Y_AC[0],
                                         &DefaultHuffmanTable_Y_AC[16],
                                         HuffmanTree_Y_AC)
        != ippStsNoErr) throw mxcpcDidntWorkOutException();
        
    if(ippiDecodeHuffmanSpecInit_JPEG_8u(&DefaultHuffmanTable_UV_DC[0],
                                         &DefaultHuffmanTable_UV_DC[16],
                                         HuffmanTree_UV_DC)
        != ippStsNoErr) throw mxcpcDidntWorkOutException();
        
    if(ippiDecodeHuffmanSpecInit_JPEG_8u(&DefaultHuffmanTable_UV_AC[0],
                                         &DefaultHuffmanTable_UV_AC[16],
                                         HuffmanTree_UV_AC)
        != ippStsNoErr) throw mxcpcDidntWorkOutException();
        
  } catch(mxcpcDidntWorkOutException) {
  
      std::free(HuffmanTree_Y_DC);
      std::free(HuffmanTree_Y_AC);
      std::free(HuffmanTree_UV_DC);
      std::free(HuffmanTree_UV_AC);
      
      throw;
    }
    
  for(i = 0; i < 64; i++) {
    QuantizationTable_Y[i]  = 0;
    QuantizationTable_UV[i] = 0;
  }
}   


mxcpcFramewiseMxPEGScanDecoderIPP::~mxcpcFramewiseMxPEGScanDecoderIPP() {

  std::free(HuffmanTree_Y_DC);
  std::free(HuffmanTree_Y_AC);
  std::free(HuffmanTree_UV_DC);
  std::free(HuffmanTree_UV_AC); 
}   



const char *mxcpcFramewiseMxPEGScanDecoderIPP
            ::getAccelerationTypeString(void) {
                    
  return("Intel Performance Primitives");
}


void mxcpcFramewiseMxPEGScanDecoderIPP::setYQuantizationTable(
                                         unsigned char *data_bytes) {

  ippiQuantInvTableInit_JPEG_8u16u(data_bytes,
                                   QuantizationTable_Y);
}

 
void mxcpcFramewiseMxPEGScanDecoderIPP::setUVQuantizationTable(
                                         unsigned char *data_bytes) {

  ippiQuantInvTableInit_JPEG_8u16u(data_bytes,
                                   QuantizationTable_UV);
}


void mxcpcFramewiseMxPEGScanDecoderIPP
      ::performScanSweep(mxcpcFramewiseMxPEGDecoder::UndecodedFrameDescriptor 
                          *frame_descriptor)                                  {
                
  // init some local data facilitating invocation of the IPP routines...
  //   done only once for an entire scan sweep, so we can do this here on the
  //   stack...
  const IppiDecodeHuffmanSpec *dc_huffman_trees[6] = { HuffmanTree_Y_DC,
                                                       HuffmanTree_Y_DC,
                                                       HuffmanTree_Y_DC, 
                                                       HuffmanTree_Y_DC,
                                                       HuffmanTree_UV_DC, 
                                                       HuffmanTree_UV_DC,                                      
                                                     };    
  const IppiDecodeHuffmanSpec *ac_huffman_trees[6] = { HuffmanTree_Y_AC,
                                                       HuffmanTree_Y_AC,
                                                       HuffmanTree_Y_AC, 
                                                       HuffmanTree_Y_AC,
                                                       HuffmanTree_UV_AC, 
                                                       HuffmanTree_UV_AC,                                      
                                                     }; 
  Ipp16s last_y_dc = 0, last_u_dc = 0, last_v_dc = 0;
  Ipp16s *last_dc_ptrs[6] = { &last_y_dc, &last_y_dc, &last_y_dc, &last_y_dc,
                              &last_u_dc, &last_v_dc
                            };
  int detected_marker = 0;
  Ipp32u prefetched_bits = 0;
  int prefetched_bit_num = 0;
  int pos = 0;
  Ipp16u *quantization_table_ptrs[6] = { &QuantizationTable_Y[0],
                                         &QuantizationTable_Y[0],
                                         &QuantizationTable_Y[0],
                                         &QuantizationTable_Y[0],
                                         &QuantizationTable_UV[0],
                                         &QuantizationTable_UV[0]
                                       };
  Ipp8u *yuv_target_ptrs[6] = { &Y_ColorValues[0],
                                &Y_ColorValues[8],
                                &Y_ColorValues[128 + 0],
                                &Y_ColorValues[128 + 8],
                                &U_ColorValues[0],
                                &V_ColorValues[0]
                              };
  int yuv_target_steps[6] = { 16,
                              16,
                              16,
                              16,
                              8,
                              8
                            };                             
  const Ipp8u *yuv_src_ptrs[3] = { &Y_ColorValues[0],
                                   &U_ColorValues[0],
                                   &V_ColorValues[0] 
                           };
  int yuv_src_steps[3] = { 16, 8, 8 };
  IppiSize yuv_rgb_roi_size = { 16, 16 };
  
  // remaining local data...
  mxcpc::u32 mini_buffer[16*16*3/4];
  mxcpc::u32 *final_copy_source,
             *final_copy_target;
  int target_row_stepping;
  IppStatus ipp_status;
  bool use_bitmask;
  int tilebits_left;
  unsigned char *tilebit_ptr, tilebit_mask;
  int x, y;
  bool next_tile_right_here;
  int i;
  
  
  // guard against missing backend
  if(!BackEnd) return;
  
    
  // initialize and decide for bitmask mode...
  tilebits_left = frame_descriptor->TileNumX * frame_descriptor->TileNumY;
  tilebit_ptr   = frame_descriptor->TileBits;
  tilebit_mask  = 128;
  x = -1;
  y = 0;
  ipp_status = ippStsNoErr;
  next_tile_right_here = true;   // default for bitmask-less mode
  if(tilebit_ptr) 
    use_bitmask = true;
  else
    use_bitmask = false;
   
    
  // iterate over all tiles (MCUs)...
  while(tilebits_left) {
  
    // advance target tile coordinates...
    x++;
    if(x == frame_descriptor->TileNumX) {
      x = 0;
      y++;
    }
  
    // if in bitmask mode, consume and honor current tile bit...
    if(use_bitmask) {
      next_tile_right_here = (*tilebit_ptr & tilebit_mask) != 0;
      tilebit_mask /= 2;
      if(!tilebit_mask) {
        tilebit_mask = 128;
        tilebit_ptr++;
      }
    }
   
    if(next_tile_right_here) {   // decode tile...
      
      // iterate over the 6 sub-MCUs (Y,Y,Y,Y,U,V)...
      for(i = 0; i < 6; i++) {
       
        if(ipp_status != ippStsNoErr) {   // decoding of previous sub-MCU caused
                                          // an error - better stop...
          BackEnd->errorEncountered();
          return;
        }
        
        ipp_status = ippiDecodeHuffman8x8_Direct_JPEG_1u16s_C1(
                      frame_descriptor->ScanBytes, &pos,
                      DecodedHuffmanCoeffs[i],
                      last_dc_ptrs[i],
                      &detected_marker,
                      &prefetched_bits, &prefetched_bit_num,
                      dc_huffman_trees[i], ac_huffman_trees[i]);
                      
        ippiDCTQuantInv8x8LS_JPEG_16s8u_C1R(DecodedHuffmanCoeffs[i],
                                            yuv_target_ptrs[i],
                                            yuv_target_steps[i],
                                            quantization_table_ptrs[i]);
      }
      
      // got all 6 sub-MCUs!
            
      // now perform colorspace conversion... 
      ippiYUV420ToRGB_8u_P3C3R(yuv_src_ptrs, yuv_src_steps, 
                               (Ipp8u *)mini_buffer,
                               16*3, yuv_rgb_roi_size);
    
      // --- finally negotiate with backend and write decoded pixel data... ---
      BackEnd->provideVideoTilePixelTarget(x, y, &final_copy_target, &target_row_stepping);
      final_copy_source = mini_buffer;
      for(i = 0; i < 16; i++) {   // 16 rows
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        *final_copy_target++ = *final_copy_source++; 
        final_copy_target += target_row_stepping;
      } 
      BackEnd->videoTileReceived();
    } 
    
    tilebits_left--;
  }
}
